#
# Copyright (c) 2013 by Takashi Sakamoto
# Copyright (C) 2005-2008 by Pieter Palmers
#
# This file is part of FFADO
# FFADO = Free FireWire (pro-)audio drivers for Linux
#
# FFADO is based upon FreeBoB.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 2 of the License, or
# (at your option) version 3 of the License.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#

# from PyQt4.QtCore import Qt
# from PyQt4.QtGui import QSizePolicy, QHBoxLayout, QVBoxLayout, QGroupBox
# from PyQt4.QtGui import QWidget, QTabWidget, QLabel, QSlider, QToolButton
from ffado.import_pyqt import *

from math import log10
from ffado.config import *

import logging
log = logging.getLogger('MAudioBeBoB')

class MAudio_BeBoB_Input_Widget(QWidget):
    def __init__(self, parent=None):
        QWidget.__init__(self, parent)
        uicLoad("ffado/mixer/maudio_bebob_input", self)

class MAudio_BeBoB_Output_Widget(QWidget):
    def __init__(self, parent=None):
        QWidget.__init__(self, parent)
        uicLoad("ffado/mixer/maudio_bebob_output", self)


class MAudio_BeBoB(QWidget):
    def __init__(self, parent=None):
        QWidget.__init__(self, parent)

    info = {
        0x0000000a: (0, "Ozonic"),
        0x00010062: (1, "FireWire Solo"),
        0x00010060: (2, "FireWire Audiophile"),
        0x00010046: (3, "FireWire 410"),
        0x00010071: (4, "FireWire 1814"),
        0x00010091: (4, "ProjectMix I/O"),
    }

    labels = (
        {
            "inputs":   ("Analog 1/2", "Analog 3/4",
                         "Stream 1/2", "Stream 3/4"),
            "mixers":   ("Mixer 1/2", "Mixer 3/4"),
            "outputs":  ("Analog 1/2", "Analog 3/4")
        },
        {
            "inputs":   ("Analog 1/2", "Digital 1/2",
                         "Stream 1/2", "Stream 3/4"),
            "mixers":   ("Mixer 1/2", "Mixer 3/4"),
            "outputs":  ("Analog 1/2", "Digital 1/2")
        },
        {
            "inputs":   ("Analog 1/2", "Digital 1/2",
                         "Stream 1/2", "Stream 3/4", "Stream 5/6"),
            "mixers":   ("Mixer 1/2", "Mixer 3/4", "Mixer 5/6", "Aux 1/2"),
            "outputs":  ("Analog 1/2", "Analog 3/4", "Digital 1/2",
                         "Headphone 1/2")
        },
        {
            "inputs":   ("Analog 1/2", "Digital 1/2",
                         "Stream 1/2", "Stream 3/4", "Stream 5/6",
                         "Stream 7/8", "Stream 9/10"),
            "mixers":   ("Mixer 1/2", "Mixer 3/4", "Mixer 5/6", "Mixer 7/8",
                         "Mixer 9/10", "Aux 1/2"),
            "outputs":  ("Analog 1/2", "Analog 3/4", "Analog 5/6", "Analog 7/8",
                         "Digital 1/2", "Headphone 1/2")
        },
        {
            "inputs":   ("Analog 1/2", "Analog 3/4", "Analog 5/6", "Analog 7/8",
                         "Stream 1/2", "Stream 3/4", "S/PDIF 1/2",
                         "ADAT 1/2", "ADAT 3/4", "ADAT 5/6", "ADAT 7/8"),
            "mixers":   ("Mixer 1/2", "Mixer 3/4", "Aux 1/2"),
            "outputs":  ("Analog 1/2", "Analog 3/4",
                         "Headphone 1/2", "Headphone 3/4")
        }
    )

    # hardware inputs and stream playbacks
    #  format: function_id/channel_idx/panning-able
    #  NOTE: function_id = channel_idx = panning-able =  labels["inputs"]
    inputs = (
        (
            (0x03, 0x04, 0x01, 0x02),
            ((0x01, 0x02), (0x01, 0x02), (0x01, 0x02), (0x01, 0x02)),
            (True, True, False, False)
        ),
        (
            (0x01, 0x02, 0x04, 0x03),
            ((0x01, 0x02), (0x01, 0x02), (0x01, 0x02), (0x01, 0x02)),
            (True, True, False, False)
        ),
        (
            (0x04, 0x05, 0x01, 0x02, 0x03),
            ((0x01, 0x02), (0x01, 0x02),
             (0x01, 0x02), (0x01, 0x02), (0x01, 0x02)),
            (True, True, False, False, False)
        ),
        (
            (0x03, 0x04, 0x02, 0x01, 0x01, 0x01, 0x01),
            ((0x01, 0x02), (0x01, 0x02), (0x01, 0x02),
             (0x01, 0x02), (0x03, 0x04), (0x05, 0x06), (0x07, 0x08)),
            (True, True, False, False, False, False, False)
        ),
        (
            (0x01, 0x02, 0x03, 0x04, 0x0a, 0x0b, 0x05, 0x06, 0x07, 0x08, 0x09),
            ((0x01, 0x02), (0x01, 0x02), (0x01, 0x02), (0x01, 0x02),
             (0x01, 0x02), (0x01, 0x02), (0x01, 0x02),
             (0x01, 0x02), (0x01, 0x02), (0x01, 0x02), (0x01, 0x02)),
            (True, True, True, True, False, False, True,
             True, True, True, True)
        )
    )

    # jack sources except for headphone
    #  format: function_id/source id
    #  NOTE: "function_id" = labels["output"] - "Headphone 1/2/3/4"
    #  NOTE: "source_id" = labels["mixer"]
    jack_src = (
        None,
        None,
        ((0x01, 0x02, 0x03),
         (0x00, 0x00, 0x00, 0x01)),
        ((0x02, 0x03, 0x04, 0x05, 0x06),
         (0x00, 0x00, 0x00, 0x00, 0x00, 0x01)),
        ((0x03, 0x04),
         (0x00, 0x01, 0x02))
    )

    # headphone sources
    #  format: sink id/source id
    #  NOTE: "source_id" = labels["mixer"]
    hp_src = (
        None,
        None,
        ((0x04,),
         (0x00, 0x01, 0x02, 0x03)),
        ((0x07,),
         (0x02, 0x03, 0x04, 0x05, 0x06, 0x07)),
        ((0x01, 0x02),
         (0x01, 0x02, 0x04))
    )

    # hardware outputs
    #  format: function id
    #  NOTE: "function_id" = labels["output"]
    outputs = (
        (0x05, 0x06),
        (0x02, 0x03),
        (0x0c, 0x0d, 0x0e, 0x0f),
        (0x0a, 0x0b, 0x0c, 0x0d, 0x0e, 0x0f),
        (0x0c, 0x0d, 0x0f, 0x10)
    )

    # Mixer inputs/outputs
    #  format: function_id/out_stereo_channel_id/in_id/in_stereo_channel_id
    #  NOTE: function_id = out_stereo_channel_id = labels["mixers"]
    #  NOTE: in_id = in_stereo_channel_id = labels["inputs"]
    mixers = (
        (
            (0x01, 0x02),
            ((0x01, 0x02), (0x01, 0x02)),
             (0x02, 0x03, 0x00, 0x01),
            ((0x01, 0x02), (0x01, 0x02), (0x01, 0x02), (0x01, 0x02))
        ),
        (
            (0x01, 0x01),
            ((0x01, 0x02), (0x03, 0x04)),
            (0x00, 0x01, 0x03, 0x02),
            ((0x01, 0x02), (0x01, 0x02), (0x01, 0x02), (0x01, 0x02))
        ),
        (
            (0x01, 0x02, 0x03, 0x04),
            ((0x01, 0x02), (0x01, 0x02), (0x01, 0x02), (0x01, 0x02)),
            (0x03, 0x04, 0x00, 0x01, 0x02),
            ((0x01, 0x02), (0x01, 0x02), (0x01, 0x02),
             (0x01, 0x02), (0x01, 0x02))
        ),
        (
            (0x01, 0x01, 0x01, 0x01, 0x01, 0x07),
            ((0x01, 0x02), (0x03, 0x04),
             (0x05, 0x06), (0x07, 0x08), (0x09, 0x0a), (0x01, 0x02)),
            (0x02, 0x03, 0x01, 0x00, 0x00, 0x00, 0x00),
            ((0x01, 0x02), (0x01, 0x02),
             (0x01, 0x02), (0x01, 0x02), (0x03, 0x04),
             (0x05, 0x06), (0x07, 0x08))
        ),
        (
            (0x01, 0x02, 0x03),
            ((0x01, 0x02), (0x01, 0x02), (0x01, 0x02)),
            (0x01, 0x01, 0x01, 0x01,
             0x02, 0x02, 0x03,
             0x04, 0x04, 0x04, 0x04),
            ((0x01, 0x02), (0x03, 0x04), (0x05, 0x06), (0x07, 0x08),
             (0x01, 0x02), (0x03, 0x04), (0x01, 0x02),
             (0x01, 0x02), (0x03, 0x04), (0x05, 0x06), (0x07, 0x08))
        )
    )

    # Aux mixer inputs/outputs
    #  format: function_id/input_id/input_stereo_channel_id
    #  NOTE: input_id = labels["inputs"]
    aux = (
        None,
        None,
        (
            0x0b,
            (0x09, 0x0a, 0x06, 0x07, 0x08),
            ((0x01, 0x02), (0x01, 0x02),
             (0x01, 0x02), (0x01, 0x02), (0x01, 0x02))
        ),
        (
            0x09,
            (0x07, 0x08, 0x06, 0x05, 0x05, 0x05, 0x05),
            ((0x01, 0x02), (0x01, 0x02), (0x01, 0x02),
             (0x01, 0x02), (0x03, 0x04), (0x05, 0x06), (0x07, 0x08))
        ),
        (
            0x0e,
            (0x13, 0x14, 0x15, 0x16, 0x11, 0x12, 0x17, 0x18, 0x19, 0x1a, 0x1b),
            ((0x01, 0x02), (0x01, 0x02), (0x01, 0x02), (0x01, 0x02),
             (0x01, 0x02), (0x01, 0x02), (0x01, 0x02),
             (0x01, 0x02), (0x01, 0x02), (0x01, 0x02), (0x01, 0x02))
        )
    )

    def getDisplayTitle(self):
        model = self.configrom.getModelId()
        return self.info[model][1]

    def buildMixer(self):
        self.Selectors = {}
        self.Pannings = {}
        self.Volumes = {}
        self.Mutes = {}
        self.Mixers = {}
        self.FW410HP = 0

        model = self.configrom.getModelId()
        if model not in self.info:
            return

        self.id = self.info[model][0]

        tabs_layout = QHBoxLayout(self)
        tabs = QTabWidget(self)

        self.addInputTab(tabs)
        self.addMixTab(tabs)
        if self.aux[self.id] is not None:
            self.addAuxTab(tabs)
        self.addOutputTab(tabs)

        tabs_layout.addWidget(tabs)


    def addInputTab(self, tabs):
        tab_input = QWidget(self)
        tabs.addTab(tab_input, "In")

        tab_input_layout = QHBoxLayout()
        tab_input.setLayout(tab_input_layout)

        in_labels = self.labels[self.id]["inputs"]
        in_ids = self.inputs[self.id][0]
        in_pan = self.inputs[self.id][2]

        for i in range(len(in_ids)):
            l_idx = self.inputs[self.id][1][i][0]
            r_idx = self.inputs[self.id][1][i][1]

            widget = MAudio_BeBoB_Input_Widget(tab_input)
            tab_input_layout.addWidget(widget)

            widget.name.setText(in_labels[i])

            self.Volumes[widget.l_sld] = (
                "/Mixer/Feature_Volume_%d" % in_ids[i], l_idx,
                widget.r_sld, r_idx,
                widget.link
            )
            self.Volumes[widget.r_sld] = (
                "/Mixer/Feature_Volume_%d" % in_ids[i], r_idx,
                widget.l_sld, l_idx,
                widget.link
            )
            self.Mutes[widget.mute] = (widget.l_sld, widget.r_sld)

            if not in_pan[i]:
                widget.l_pan.setDisabled(True)
                widget.r_pan.setDisabled(True)
            else:
                self.Pannings[widget.l_pan] = (
                    "/Mixer/Feature_LRBalance_%d" % in_ids[i],
                    l_idx
                )
                self.Pannings[widget.r_pan] = (
                    "/Mixer/Feature_LRBalance_%d" % in_ids[i],
                    r_idx
                )

        tab_input_layout.addStretch()


    def addMixTab(self, tabs):
        tab_mix = QWidget(self)
        tabs.addTab(tab_mix, "Mix")

        tab_layout = QHBoxLayout()
        tab_mix.setLayout(tab_layout)

        in_labels = self.labels[self.id]["inputs"]
        in_idxs = self.inputs[self.id][0]

        mix_labels = self.labels[self.id]["mixers"]
        mix_idxs = self.mixers[self.id][0]

        for i in range(len(mix_idxs)):
            if mix_labels[i] == 'Aux 1/2':
                continue

            grp = QGroupBox(tab_mix)
            grp_layout = QVBoxLayout()
            grp.setLayout(grp_layout)
            tab_layout.addWidget(grp)

            label = QLabel(grp)
            grp_layout.addWidget(label)

            label.setText(mix_labels[i])
            label.setAlignment(Qt.AlignCenter)
            label.setSizePolicy(QSizePolicy.Expanding, QSizePolicy.Minimum)

            for j in range(len(in_idxs)):
                mix_in_id = self.mixers[self.id][2][j]

                button = QToolButton(grp)
                grp_layout.addWidget(button)

                button.setSizePolicy(QSizePolicy.Expanding, QSizePolicy.Minimum)
                button.setText('%s In' % in_labels[j])
                button.setCheckable(True)

                self.Mixers[button] = (
                    "/Mixer/EnhancedMixer_%d" % mix_idxs[i],
                    mix_in_id, j, i
                )

            grp_layout.addStretch()
        tab_layout.addStretch()

    def addAuxTab(self, tabs):
        #local functions
        def addLinkButton(parent, layout):
            button = QToolButton(parent)
            layout.addWidget(button)
            button.setSizePolicy(QSizePolicy.Expanding, QSizePolicy.Minimum)
            button.setText('Link')
            button.setCheckable(True)
            return button
        def addMuteButton(parent, layout):
            button = QToolButton(parent)
            layout.addWidget(button)
            button.setSizePolicy(QSizePolicy.Expanding, QSizePolicy.Minimum)
            button.setText('Mute')
            button.setCheckable(True)
            return button

        # local processing
        tab_aux = QWidget(self)
        tabs.addTab(tab_aux, "Aux")

        layout = QHBoxLayout()
        tab_aux.setLayout(layout)

        aux_label = self.labels[self.id]["mixers"][-1]
        aux_id = self.aux[self.id][0]

        aux_in_labels = self.labels[self.id]["inputs"]
        aux_in_ids = self.aux[self.id][1]

        for i in range(len(aux_in_ids)):
            in_ch_l = self.aux[self.id][2][i][0]
            in_ch_r = self.aux[self.id][2][i][1]

            grp = QGroupBox(tab_aux)
            grp_layout = QVBoxLayout()
            grp.setLayout(grp_layout)
            layout.addWidget(grp)

            label = QLabel(grp)
            grp_layout.addWidget(label)
            label.setText("%s\nIn" % aux_in_labels[i])
            label.setAlignment(Qt.AlignCenter)

            grp_sld = QGroupBox(grp)
            grp_sld_layout = QHBoxLayout()
            grp_sld.setLayout(grp_sld_layout)
            grp_layout.addWidget(grp_sld)

            l_sld = QSlider(grp_sld)
            grp_sld_layout.addWidget(l_sld)
            r_sld = QSlider(grp_sld)
            grp_sld_layout.addWidget(r_sld)

            button = addLinkButton(grp, grp_layout)
            self.Volumes[l_sld] = (
                "/Mixer/Feature_Volume_%d" % aux_in_ids[i], in_ch_l,
                r_sld, in_ch_r,
                button
            )
            self.Volumes[r_sld] = (
                "/Mixer/Feature_Volume_%d" % aux_in_ids[i], in_ch_r,
                l_sld, in_ch_l,
                button
            )

            button = addMuteButton(grp, grp_layout)
            self.Mutes[button] = (l_sld, r_sld)

        grp = QGroupBox(tab_aux)
        grp_layout = QVBoxLayout()
        grp.setLayout(grp_layout)
        layout.addWidget(grp)

        label = QLabel(grp)
        grp_layout.addWidget(label)
        label.setText("%s\nOut" % aux_label)
        label.setAlignment(Qt.AlignCenter)

        grp_sld = QGroupBox(grp)
        grp_sld_layout = QHBoxLayout()
        grp_sld.setLayout(grp_sld_layout)
        grp_layout.addWidget(grp_sld)

        l_sld = QSlider(grp_sld)
        grp_sld_layout.addWidget(l_sld)
        r_sld = QSlider(grp_sld)
        grp_sld_layout.addWidget(r_sld)

        button = addLinkButton(grp, grp_layout)
        self.Volumes[l_sld] = (
            "/Mixer/Feature_Volume_%d" % aux_id, 1,
            r_sld, 2,
            button
        )
        self.Volumes[r_sld] = (
            "/Mixer/Feature_Volume_%d" % aux_id, 2,
            l_sld, 1,
            button
        )

        button = addMuteButton(grp, grp_layout)
        self.Mutes[button] = (l_sld, r_sld)

        layout.addStretch()

    def addOutputTab(self, tabs):
        tab_out = QWidget(self)
        tabs.addTab(tab_out, "Out")

        layout = QHBoxLayout()
        tab_out.setLayout(layout)

        out_labels = self.labels[self.id]["outputs"]
        out_ids = self.outputs[self.id]

        mixer_labels = self.labels[self.id]["mixers"]

        if self.jack_src[self.id] is None:
            for i in range(len(out_ids)):
                label = QLabel(tab_out)
                layout.addWidget(label)
                label.setText("%s Out is fixed to %s Out" %
                              (mixer_labels[i], out_labels[i]))
            return

        mixer_ids = self.jack_src[self.id][1]


        for i in range(len(out_ids)):
            out_label = out_labels[i]
            if out_label.find('Headphone') >= 0:
                continue

            out_id = self.jack_src[self.id][0][i]

            widget = MAudio_BeBoB_Output_Widget(tab_out)
            layout.addWidget(widget)

            widget.name.setText(out_label)

            self.Volumes[widget.l_sld] = (
                "/Mixer/Feature_Volume_%d" % out_ids[i], 1,
                widget.r_sld, 2,
                widget.link
            )
            self.Volumes[widget.r_sld] = (
                "/Mixer/Feature_Volume_%d" % out_ids[i], 2,
                widget.l_sld, 1,
                widget.link
            )
            self.Mutes[widget.mute] = (widget.l_sld, widget.r_sld)

            self.Selectors[widget.cmb_src] = ("/Mixer/Selector_%d" % out_id, )

            for j in range(len(mixer_ids)):
                if i != j and j != len(mixer_ids) - 1:
                    continue
                widget.cmb_src.addItem("%s Out" % mixer_labels[j], mixer_ids[j])

        # add headphone
        hp_idx = 0
        for i in range(len(out_ids)):
            out_label = out_labels[i]
            if out_label.find('Headphone') < 0:
                continue

            hp_label = self.labels[self.id]["outputs"][i]
            hp_id = self.hp_src[self.id][0][hp_idx]
            hp_idx += 1

            mixer_labels = self.labels[self.id]["mixers"]

            widget = MAudio_BeBoB_Output_Widget(tab_out)
            layout.addWidget(widget)

            widget.name.setText(hp_label)

            mixer_labels = self.labels[self.id]["mixers"]
            mixer_ids = self.mixers[self.id][0]

            self.Volumes[widget.l_sld] = (
                "/Mixer/Feature_Volume_%d" % out_ids[i], 1,
                widget.r_sld, 2,
                widget.link
            )
            self.Volumes[widget.r_sld] = (
                "/Mixer/Feature_Volume_%d" % out_ids[i], 2,
                widget.l_sld, 1,
                widget.link
            )
            self.Mutes[widget.mute] = (widget.l_sld, widget.r_sld)

            for i in range(len(mixer_ids)):
                widget.cmb_src.addItem("%s Out" % mixer_labels[i], mixer_ids[i])

            if self.id != 3:
                self.Selectors[widget.cmb_src] = ("/Mixer/Selector_%d" % hp_id, )
            else:
                widget.cmb_src.activated.connect(self.update410HP)
                self.FW410HP = widget.cmb_src

        layout.addStretch()

    def initValues(self):
        for ctl, params in list(self.Selectors.items()):
            path = params[0]
            state = self.hw.getDiscrete(path)
            ctl.setCurrentIndex(state)
            ctl.activated.connect(self.updateSelector)

        #       Right - Center - Left
        # 0x8000 - 0x0000 - 0x0001 - 0x7FFE
        #        ..., -1, 0, +1, ...
        for ctl, params in list(self.Pannings.items()):
            path = params[0]
            idx = params[1]
            curr = self.hw.getContignuous(path, idx)
            state = -(curr / 0x7FFE) * 50 + 50
            ctl.setValue(int(state))
            ctl.valueChanged.connect(self.updatePanning)

        for ctl, params in list(self.Volumes.items()):
            path = params[0]
            idx = params[1]
            p_idx = params[3]
            link = params[4]

            db = self.hw.getContignuous(path, idx)
            vol = self.db2vol(db)
            ctl.setValue(int(vol))
            ctl.valueChanged.connect(self.updateVolume)

            # to activate link button, a pair is checked twice, sign...
            pair_db = self.hw.getContignuous(path, p_idx)
            if pair_db == db:
                link.setChecked(True)

        for ctl, params in list(self.Mutes.items()):
            ctl.clicked.connect(self.updateMute)

        for ctl, params in list(self.Mixers.items()):
            path = params[0]
            in_id = params[1]
            mix_in_idx = params[2]
            mix_out_idx = params[3]
            in_ch_l = self.mixers[self.id][3][mix_in_idx][0]
            out_ch_l = self.mixers[self.id][1][mix_out_idx][0]
            # see /libffado/src/bebob/bebob_mixer.cpp
            mux_id = self.getMultiplexedId(in_id, in_ch_l, out_ch_l)
            curr = self.hw.getContignuous(path, mux_id)
            if curr == 0:
                state = True
            else:
                state = False
            ctl.setChecked(state)
            ctl.clicked.connect(self.updateMixer)

        if self.id == 3:
            self.read410HP()

    # helper functions
    def vol2db(self, vol):
        return (log10(vol + 1) - 2) * 16384

    def db2vol(self, db):
        return pow(10, db / 16384 + 2) - 1

    def getMultiplexedId(self, in_id, in_ch_l, out_ch_l):
        # see /libffado/src/bebob/bebob_mixer.cpp
        return (in_id << 8) | (in_ch_l << 4) | (out_ch_l << 0)

    def updateSelector(self, state):
        sender = self.sender()
        path = self.Selectors[sender][0]
        log.debug("set %s to %d" % (path, state))
        self.hw.setDiscrete(path, state)

    def updatePanning(self, state):
        sender = self.sender()
        path = self.Pannings[sender][0]
        idx = self.Pannings[sender][1]
        value = (state - 50) * 0x7FFE / -50
        log.debug("set %s for %d to %d(%d)" % (path, idx, value, state))
        self.hw.setContignuous(path, value, idx)

    def updateVolume(self, vol):
        sender = self.sender()
        path = self.Volumes[sender][0]
        idx = self.Volumes[sender][1]
        pair = self.Volumes[sender][2]
        link = self.Volumes[sender][4]

        db = self.vol2db(vol)
        log.debug("set %s for %d to %d(%d)" % (path, idx, db, vol))
        self.hw.setContignuous(path, db, idx)

        if link.isChecked():
            pair.setValue(vol)

    # device remeber gain even if muted
    def updateMute(self, state):
        sender = self.sender()
        l_sld = self.Mutes[sender][0]
        r_sld = self.Mutes[sender][1]

        if state:
            db = 0x8000
        else:
            db = 0x0000
        for w in [l_sld, r_sld]:
            path = self.Volumes[w][0]
            self.hw.setContignuous(path, db)
            w.setDisabled(state)

    def updateMixer(self, checked):
        if checked:
            state = 0x0000
        else:
            state = 0x8000

        sender = self.sender()
        path = self.Mixers[sender][0]
        in_id = self.Mixers[sender][1]
        mix_in_idx = self.Mixers[sender][2]
        mix_out_idx = self.Mixers[sender][3]
        in_ch_l = self.mixers[self.id][3][mix_in_idx][0]
        out_ch_l = self.mixers[self.id][1][mix_out_idx][0]

        mux_id = self.getMultiplexedId(in_id, in_ch_l, out_ch_l)

        log.debug("set %s for 0x%04X(%d/%d/%d) to %d" %
                  (path, mux_id, in_id, in_ch_l, out_ch_l, state))
        self.hw.setContignuous(path, state, mux_id)

    def read410HP(self):
        path = "/Mixer/Selector_7"
        sel = self.hw.getDiscrete(path)
        if sel > 0:
            enbl = 5
        else:
            enbl = -1

        path = "/Mixer/EnhancedMixer_7"
        for i in range(5):
            in_id = 0
            in_ch_l = i * 2 + 1
            out_ch_l = 1
            mux_id = self.getMultiplexedId(in_id, in_ch_l, out_ch_l)
            state = self.hw.getContignuous(path, mux_id)
            if enbl < 0 and state == 0:
                enbl = i
            else:
                self.hw.setContignuous(path, 0x8000, mux_id)
        # if inconsistency between Selector and Mixer, set AUX as default
        if enbl == -1:
            self.hw.setDiscrete('/Mixer/Selector_7', 1)
            enbl = 5

        self.FW410HP.setCurrentIndex(enbl)

    def update410HP(self, state):
        # each output from mixer can be multiplexed in headphone
        # but here they are exclusive because of GUI simpleness, sigh...
        path = "/Mixer/EnhancedMixer_7"
        for i in range(5):
            in_id = 0
            in_ch_l = i * 2 + 1
            out_ch_l = 1
            mux_id = self.getMultiplexedId(in_id, in_ch_l, out_ch_l)
            if i == state:
                value = 0x0000
            else:
                value = 0x8000
            self.hw.setContignuous(path, value, mux_id)

        # Mixer/Aux is selectable exclusively
        path = "/Mixer/Selector_7"
        sel = (state == 5)
        self.hw.setDiscrete(path, sel)




#
# Copyright (C) 2005-2008 by Pieter Palmers
#               2007-2008 by Arnold Krille
#               2013 by Philippe Carriere
#
# This file is part of FFADO
# FFADO = Free FireWire (pro-)audio drivers for Linux
#
# FFADO is based upon FreeBoB.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#

from ffado.config import * #FFADO_VERSION, FFADO_DBUS_SERVER, FFADO_DBUS_BASEPATH

# from PyQt4.QtGui import QFrame, QWidget, QTabWidget, QVBoxLayout, QMainWindow, QIcon, QAction, qApp, QStyleOptionTabWidgetFrame, QFileDialog
# from PyQt4.QtCore import QTimer, pyqtSignal
from ffado.import_pyqt import *

from ffado.dbus_util import *
from ffado.registration import *

from ffado.configuration import *

from ffado.mixer.globalmixer import GlobalMixer
from ffado.mixer.dummy import Dummy

import sys
import time
import importlib

import logging
log = logging.getLogger('panelmanager')

use_generic = False
try:
    from mixer_generic import *
    log.info("The generic mixer is found, seems to be a developer using ffadomixer...")
except ImportError:
    pass
else:
    use_generic = True

# pseudo-guid
GUID_GENERIC_MIXER = 0

FILE_VERSION = '0.1'

class HLine( QFrame ):
    def __init__( self, parent ):
        QFrame.__init__( self, parent )
        self.setFrameShape( QFrame.HLine )
        self.setLineWidth( 2 )
        self.setMinimumHeight( 10 )

class PanelManagerStatus(QWidget):
    def __init__(self, parent):
        QWidget.__init__(self,parent)
        uicLoad("ffado/panelmanagerstatus", self)

class OwnTabWidget(QTabWidget):
    def __init__(self,parent):
        QTabWidget.__init__(self,parent)

    def tabInserted(self,index):
        self.checkTabBar()

    def tabRemoved(self,index):
        self.checkTabBar()

    def checkTabBar(self):
        if self.count()<2:
            self.tabBar().hide()
        else:
            self.tabBar().show()

class PanelManager(QWidget):
    connectionLost = pyqtSignal(name='connectionLost')
    def __init__(self, parent, devmgr=None):
        QWidget.__init__(self, parent)
        self.setObjectName("PanelManager")
        self.parent = parent

        # maps a device GUID to a QT panel
        self.panels = {}

        # a layout for ourselves
        self.layout = QVBoxLayout(self)

        # the tabs
        self.tabs = OwnTabWidget(self)
        self.tabs.hide()
        self.layout.addWidget(self.tabs)

        # a dialog that is shown during update
        self.status = PanelManagerStatus(self)
        self.layout.addWidget(self.status)
        self.status.show()

        self.devices = DeviceList( SYSTEM_CONFIG_FILE )
        self.devices.updateFromFile( USER_CONFIG_FILE )

        if devmgr is not None:
            self.setManager(devmgr)

    def __del__(self):
        print("PanelManager.__del__()")
        self.polltimer.stop()

    def setManager(self,devmgr):
        self.devmgr = devmgr
        self.devmgr.registerPreUpdateCallback(self.devlistPreUpdate)
        self.devmgr.registerPostUpdateCallback(self.devlistPostUpdate)
        self.devmgr.registerUpdateCallback(self.devlistUpdate)
        self.devmgr.registerDestroyedCallback(self.devmgrDestroyed)
        # create a timer to poll the panels
        self.polltimer = QTimer()
        self.polltimer.timeout.connect(self.pollPanels)
        self.polltimer.start( POLL_SLEEP_TIME_MSEC )

        # create a timer to initialize the panel after the main form is shown
        # since initialization can take a while
        QTimer.singleShot( POLL_SLEEP_TIME_MSEC, self.updatePanels )

        # live check timer
        self.alivetimer = QTimer()
        self.alivetimer.timeout.connect(self.commCheck)
        self.alivetimer.start( 2000 )

    def count(self):
        return self.tabs.count()

    def pollPanels(self):
        #log.debug("PanelManager::pollPanels()")
        # only when not modifying the tabs
        try:
            if self.tabs.isEnabled():
                for guid in self.panels.keys():
                    w = self.panels[guid]
                    for child in w.children():
                        #log.debug("poll child %s,%s" % (guid,child))
                        if 'polledUpdate' in dir(child):
                            child.polledUpdate()
        except:
            log.error("error in pollPanels")
            self.commCheck()

    def devlistPreUpdate(self):
        log.debug("devlistPreUpdate")
        self.tabs.setEnabled(False)
        self.tabs.hide()
        self.status.lblMessage.setText("Bus reconfiguration in progress, please wait...")
        self.status.show()
        #self.statusBar().showMessage("bus reconfiguration in progress...", 5000)

    def devlistPostUpdate(self):
        log.debug("devlistPostUpdate")
        # this can fail if multiple busresets happen in fast succession
        ntries = 10
        while ntries > 0:
            try:
                self.updatePanels()
                return
            except:
                log.debug("devlistPostUpdate failed (%d)" % ntries)
                for guid in self.panels.keys():
                    w = self.panels[guid]
                    del self.panels[guid] # remove from the list
                    idx = self.tabs.indexOf(w)
                    self.tabs.removeTab(idx)
                    del w # GC might also take care of that

                ntries = ntries - 1
                time.sleep(2) # sleep a few seconds

        log.debug("devlistPostUpdate failed completely")
        self.tabs.setEnabled(False)
        self.tabs.hide()
        self.status.lblMessage.setText("Error while reconfiguring. Please restart ffadomixer.")
        self.status.show()


    def devlistUpdate(self):
        log.debug("devlistUpdate")

    def devmgrDestroyed(self):
        log.debug("devmgrDestroyed")
        self.alivetimer.stop()
        self.tabs.setEnabled(False)
        self.tabs.hide()
        self.status.lblMessage.setText("DBUS server was shut down, please restart it and restart ffadomixer...")
        self.status.show()

    def commCheck(self):
        try:
            nbDevices = self.devmgr.getNbDevices()
        except:
            log.error("The communication with ffado-dbus-server was lost.")
            self.tabs.setEnabled(False)
            self.polltimer.stop()
            self.alivetimer.stop()
            keys = self.panels.keys()
            for panel in keys:
                w = self.panels[panel]
                del self.panels[panel]
                w.deleteLater()
            self.connectionLost.emit()

    def removePanel(self, guid):
        print( "Removing widget for device" + guid )
        w = self.panels[guid]
        del self.panels[guid] # remove from the list
        idx = self.tabs.indexOf(w)
        self.tabs.removeTab(idx)
        w.deleteLater()
        self.parent.editmenu.removeAction(self.parent.devices[guid])
        self.parent.devices.pop(guid, None)

    def addPanel(self, idx):
        path = self.devmgr.getDeviceName(idx)
        log.debug("Adding device %d: %s" % (idx, path))

        if ffado.config.bypassdbus:
            cfgrom = ConfigRomInterface(FFADO_DBUS_SERVER, path)
        else:
            cfgrom = ConfigRomInterface(FFADO_DBUS_SERVER, FFADO_DBUS_BASEPATH+'/DeviceManager/'+path)
        vendorId = cfgrom.getVendorId()
        modelId = cfgrom.getModelId()
        unitVersion = cfgrom.getUnitVersion()
        guid = cfgrom.getGUID()
        vendorName = cfgrom.getVendorName()
        modelName = cfgrom.getModelName()
        log.debug(" Found (%s, %X, %X) %s %s" % (str(guid), vendorId, modelId, vendorName, modelName))

        # check whether this has already been registered at ffado.org
        if not ffado.config.bypassdbus:
            # As of June 2020, don't offer the registration option as the
            # statistics are no longer useful.  Coincidently, a move to
            # different website infrastructure June 2020 would necessitate
            # some work to reimplement the interfaces used by the
            # registration process, which doesn't seem worthwhile.
            #
            # reg = ffado_registration(FFADO_VERSION, int(guid, 16),
            #                              vendorId, modelId,
            #                              vendorName, modelName)
            # reg.check_for_registration()

            # The MOTU devices use unitVersion to differentiate models.  For
            # the moment though we don't need to know precisely which model
            # we're using beyond it being a pre-mark3 (modelId=0) or mark3
            # (modelId=1) device.
            if vendorId == 0x1f2:
                # All MOTU devices with a unit version of 0x15 or greater are
                # mark3 devices
                if (unitVersion >= 0x15):
                    modelId = 0x00000001
                else:
                    modelId = 0x00000000

            # The RME devices use unitVersion to differentiate models. 
            # Therefore in the configuration file we use the config file's
            # modelid field to store the unit version.  As a result we must
            # override the modelId with the unit version here so the correct
            # configuration file entry (and hense mixer widget) is identified.
            if vendorId == 0xa35:
                modelId = unitVersion;

        dev = self.devices.getDeviceById( vendorId, modelId )

        w = QWidget( )
        l = QVBoxLayout( w )

        # create a control object
        hw = ControlInterface(FFADO_DBUS_SERVER, FFADO_DBUS_BASEPATH+'/DeviceManager/'+path)
        clockselect = ClockSelectInterface( FFADO_DBUS_SERVER, FFADO_DBUS_BASEPATH+"/DeviceManager/"+path )
        samplerateselect = SamplerateSelectInterface( FFADO_DBUS_SERVER, FFADO_DBUS_BASEPATH+"/DeviceManager/"+path )
        streamingstatus = StreamingStatusInterface( FFADO_DBUS_SERVER, FFADO_DBUS_BASEPATH+"/DeviceManager/"+path )
        nickname = TextInterface( FFADO_DBUS_SERVER, FFADO_DBUS_BASEPATH+"/DeviceManager/"+path+"/Generic/Nickname" )

        #
        # Generic elements for all mixers follow here:
        #
        globalmixer = GlobalMixer( w )
        globalmixer.configrom = cfgrom
        globalmixer.clockselect = clockselect
        globalmixer.samplerateselect = samplerateselect
        globalmixer.streamingstatus = streamingstatus
        globalmixer.nickname = nickname
        globalmixer.hw = hw
        globalmixer.initValues()
        l.addWidget( globalmixer, 1 )

        #
        # Line to separate
        #
        l.addWidget( HLine( w ) )

        #
        # Specific (or dummy) mixer widgets get loaded in the following
        #
        found = False
        if 'mixer' in dev and dev['mixer'] != None:
            mixerapp = dev['mixer']
            try:
                mixer_module = importlib.import_module("ffado.mixer.%s" % mixerapp.lower())
                mixerwidget = getattr(mixer_module, mixerapp)(w)
                found = True
            except ImportError:
                log.debug("bypassdbus set, %s module not available: ignored" % mixerapp.lower())

        if not found:
            mixerwidget = Dummy( w )
            mixerapp = modelName+" (Dummy)"

        #
        # The same for all mixers
        #
        l.addWidget( mixerwidget, 10 )
        mixerwidget.configrom = cfgrom
        mixerwidget.clockselect = clockselect
        mixerwidget.samplerateselect = samplerateselect
        mixerwidget.streamingstatus = streamingstatus
        mixerwidget.nickname = nickname
        mixerwidget.hw = hw
        if 'buildMixer' in dir(mixerwidget):
            mixerwidget.buildMixer()
        if 'initValues' in dir(mixerwidget):
            mixerwidget.initValues()
        if 'getDisplayTitle' in dir(mixerwidget):
            title = mixerwidget.getDisplayTitle()
        else:
            title = mixerapp

        mixer_icon = UIDIR + '/ffado/mixer/' + vendorName.replace(" ", "_").lower() + '.png'
        if os.path.exists(mixer_icon) :
            globalmixer.lblName.setPixmap(QPixmap(mixer_icon))
            globalmixer.lblName.setToolTip(title)
            globalmixer.lblName.show()
        else :
            globalmixer.setName(title)

        self.tabs.addTab( w, title )
        self.panels[guid] = w

        if 'onSamplerateChange' in dir(mixerwidget):
          log.debug("Updating Mixer on samplerate change required")
          globalmixer.onSamplerateChange = mixerwidget.onSamplerateChange

        w.gmixSaveSetgs = globalmixer.saveSettings
        w.gmixReadSetgs = globalmixer.readSettings
        if 'saveSettings' in dir(mixerwidget):
          w.smixSaveSetgs = mixerwidget.saveSettings
          self.parent.saveaction.setEnabled(True)

        if 'readSettings' in dir(mixerwidget):
          w.smixReadSetgs = mixerwidget.readSettings
          self.parent.openaction.setEnabled(True)

        self.parent.devices[guid] = QAction(QIcon.fromTheme("audio-card"), str(title), self.parent)
        # Ensure a standard type is passed to setDate() so a UserType is not
        # adopted by the QVariant which underpins the QAction data.  "guid"
        # is a dbus.String object; it will be stored as a UserType if passed
        # directly, which makes it difficult to extract in setTabVisible().
        self.parent.devices[guid].setData(str(guid))
        self.parent.editmenu.addAction(self.parent.devices[guid])
        self.parent.devices[guid].triggered.connect(self.setTabVisible)
    
    def setTabVisible(self) :
        action = self.sender()
        # Extract the action data and store as a dbus.String type so 
        # it is usable as a key into self.panels[].
        panel_key = dbus.String(action.data().toString() if ffado_pyqt_version == 4 else action.data())
        self.tabs.setCurrentIndex(self.tabs.indexOf(self.panels[panel_key]))

    def displayPanels(self):
        # if there is no panel, add the no-device message
        # else make sure it is not present
        if self.count() == 0:
            self.tabs.hide()
            self.tabs.setEnabled(False)
            self.status.lblMessage.setText("No supported device found.")
            self.status.show()
            #self.statusBar().showMessage("No supported device found.", 5000)
        else:
            # Hide the status widget before showing the panel tab to prevent
            # the panel manager's vertical size including that of the status
            # widget.  For some reason, hiding the status after showing the
            # tabs does not cause a recalculation of the panel manager's size,
            # and the window ends up being larger than it needs to be.
            self.status.hide()
            self.tabs.show()
            self.tabs.setEnabled(True)
            #self.statusBar().showMessage("Configured the mixer for %i devices." % self.tabs.count())
            if use_generic:
                #
                # Show the generic (development) mixer if it is available
                #
                w = GenericMixer( devmgr.bus, FFADO_DBUS_SERVER, mw )
                self.tabs.addTab( w, "Generic Mixer" )
                self.panels[GUID_GENERIC_MIXER] = w
    
    def updatePanels(self):
        log.debug("PanelManager::updatePanels()")
        nbDevices = self.devmgr.getNbDevices()
        #self.statusBar().showMessage("Reconfiguring the mixer panels...")

        # list of panels present
        guids_with_tabs = self.panels.keys()

        # build list of guids on the bus now
        guids_present = []
        guid_indexes = {}
        for idx in range(nbDevices):
            path = self.devmgr.getDeviceName(idx)
            if ffado.config.bypassdbus:
                cfgrom = ConfigRomInterface(FFADO_DBUS_SERVER, path)
            else:
                cfgrom = ConfigRomInterface(FFADO_DBUS_SERVER, FFADO_DBUS_BASEPATH+'/DeviceManager/'+path)
            guid = cfgrom.getGUID()
            guids_present.append(guid)
            guid_indexes[guid] = idx

        # figure out what to remove
        # the special panel (generic)
        # that has (pseudo-)GUID 0
        # is also automatically removed
        to_remove = []
        for guid in guids_with_tabs:
            if not guid in guids_present:
                to_remove.append(guid)
                log.debug("going to remove %s" % str(guid))
            else:
                log.debug("going to keep %s" % str(guid))

        # figure out what to add
        to_add = []
        for guid in guids_present:
            if not guid in guids_with_tabs:
                to_add.append(guid)
                log.debug("going to add %s" % str(guid))

        # update the widget
        for guid in to_remove:
            self.removePanel(guid)

        for guid in to_add:
            # retrieve the device manager index
            idx = guid_indexes[guid]
            self.addPanel(idx)

        self.displayPanels()

    def refreshPanels(self):
        log.debug("PanelManager::refreshPanels()")
        nbDevices = self.devmgr.getNbDevices()
        #self.statusBar().showMessage("Reconfiguring the mixer panels...")

        # list of panels present
        guids_with_tabs = self.panels.keys()

        # build list of guids on the bus now
        guid_indexes = {}
        for idx in range(nbDevices):
            path = self.devmgr.getDeviceName(idx)
            cfgrom = ConfigRomInterface(FFADO_DBUS_SERVER, FFADO_DBUS_BASEPATH+'/DeviceManager/'+path)
            guid = cfgrom.getGUID()
            guid_indexes[guid] = idx

        # remove/create the widget
        for guid in guids_with_tabs:
            self.removePanel(guid)
            idx = guid_indexes[guid]
            self.addPanel(idx)

        self.displayPanels()

    def saveSettings(self):
        saveString = []
        saveString.append('<?xml version="1.0" encoding="UTF-8"?>\n')
        saveString.append('<fileversion>\n')
        saveString.append('  <major>\n')
        saveString.append('    ' + str(FILE_VERSION).split('.')[0] + '\n')
        saveString.append('  </major>\n')
        saveString.append('  <minor>\n')
        saveString.append('    ' + str(FILE_VERSION).split('.')[1] + '\n')
        saveString.append('  </minor>\n')
        saveString.append('</fileversion>\n')
        saveString.append('<ffadoversion>\n')
        saveString.append('  <major>\n')
        saveString.append('    ' + str(str(FFADO_VERSION).split('-')[0]).split('.')[0] + '\n')
        saveString.append('  </major>\n')
        saveString.append('  <minor>\n')
        saveString.append('    ' + str(str(FFADO_VERSION).split('-')[0]).split('.')[1] + '\n')
        saveString.append('  </minor>\n')
        saveString.append('</ffadoversion>\n')
        for guid in self.panels.keys():
          saveString.append('<device>\n')
          saveString.append('  <guid>\n')
          saveString.append('    ' + str(guid) + '\n')
          saveString.append('  </guid>\n')        
          w = self.panels[guid]
          indent = "  "
          saveString.extend(w.gmixSaveSetgs(indent))
          if 'smixSaveSetgs' in dir(w):
              saveString.extend(w.smixSaveSetgs(indent))
          saveString.append('</device>\n')
        # file saving
        savefilename = QFileDialog.getSaveFileName(self, 'Save File', os.getenv('HOME'))
        if isinstance(savefilename, tuple): # newer PyQt5
            savefilename = savefilename[0]
        try:
          f = open(savefilename, 'w')
        except IOError:
          print( "Unable to open save file" )
          return
        for s in saveString:
          f.write(s)
        f.close()

    def readSettings(self):
        readfilename = QFileDialog.getOpenFileName(self, 'Open File', os.getenv('HOME'))
        if isinstance(readfilename, tuple): # newer PyQt5
            readfilename = readfilename[0]
        try:
          f = open(readfilename, 'r')
        except IOError:
          print( "Unable to open file" )
          return
        log.debug("Opening file %s" % readfilename)
        # discard useless whitespace characters
        readString = []
        for line in f:
          readString.append(" ".join(str(line).split()))
        f.close()
        # Check it is a compatible "FFADO" file
        # It must start with the <?xml ... tag as the first string
        if readString[0].find("<?xml") == -1:
            print( "Not an xml data file" )
            return
        # Then there must be a file version tag somewhere in the file
        try:
            idx = readString.index('<fileversion>')
        except Exception:
            print( "Data file should contain the version tag" )
            return
        if readString[idx+1].find("<major>") == -1:
            print( "Incompatible versioning of the file" )
        if readString[idx+3].find("</major>") == -1:
            print( "Not a valid xml file" )
        if readString[idx+4].find("<minor>") == -1:
            print( "Incompatible versioning of the file" )
        if readString[idx+6].find("</minor>") == -1:
            print( "Not a valid xml file" )
        version_major = readString[idx+2]
        version =  version_major + '.' + readString[idx+5]
        log.debug("File version: %s" % version)
        # File version newer than present
        if int(version_major) > int(str(FILE_VERSION).split('.')[0]):
            print( "File version is too recent: you should upgrade your FFADO installation" )
            return
        # FIXME At a time it will be necessary to detect if an older major version is detected
        #
        # It looks like useless to check for the FFADO version
        # Add something here if you would like so
        #
        # Now search for devices
        nd = readString.count('<device>');
        n  = readString.count('</device>');
        if n != nd:
            print( "Not a regular xml file: opening device tag must match closing ones" )
            return
        while nd > 0:
          idxb = readString.index('<device>')
          idxe = readString.index('</device>')
          if idxe < idxb+1:
            print( "Not a regular xml file: data must be enclosed between a <device> and </device> tag" )
            return
          stringDev = []
          for s in readString[idxb:idxe]:
            stringDev.append(s)
          # determine the device guid
          try:
              idx = stringDev.index('<guid>')
          except Exception:
              print( "Device guid not found" )
              return
          guid = stringDev[idx+1]
          log.debug("Device %s found" % guid)

          if guid in self.panels:
              w = self.panels[guid]
              w.gmixReadSetgs(stringDev)
              if 'smixReadSetgs' in dir(w):
                w.smixReadSetgs(stringDev)
              log.debug("Settings changed for device %s" % guid)
          else:
              log.debug("Device %s not present; settings ignored" % guid)

          del stringDev[:]
          del readString[idxb:idxe]
          nd -= 1
   
# vim: et



#
# Copyright (C) 2005-2008 by Pieter Palmers
#
# This file is part of FFADO
# FFADO = Free FireWire (pro-)audio drivers for Linux
#
# FFADO is based upon FreeBoB.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 2 of the License, or
# (at your option) version 3 of the License.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#

# from PyQt4.QtCore import Qt, QTimer
# from PyQt4.QtGui import QWidget, QHBoxLayout, QVBoxLayout
# from PyQt4.QtGui import QGroupBox, QTabWidget, QLabel
# from PyQt4.QtGui import QPushButton, QToolButton, QSpacerItem, QSizePolicy
from ffado.import_pyqt import *

from ffado.config import *
import logging
log = logging.getLogger('audiofire')

class AfMonitorWidget(QWidget):
    def __init__(self,parent = None):
        QWidget.__init__(self,parent)
        uicLoad("ffado/mixer/audiofire_strip", self)

class AfSettingsWidget(QWidget):
    def __init__(self,parent = None):
        QWidget.__init__(self,parent)
        uicLoad("ffado/mixer/audiofire_settings", self)

class AudioFire(QWidget):
    def __init__(self,parent = None):
        QWidget.__init__(self,parent)
        log.debug("Init AudioFire mixer window")

    def getDisplayTitle(self):
        modelId = self.configrom.getModelId()
        if modelId == 0x0AF2:
            return "AudioFire2"
        if modelId == 0x0AF4:
            return "AudioFire4"
        if modelId == 0x0AF8:
            return "AudioFire8"
        if modelId == 0x0AF9:
            return "AudioFirePre8"
        if modelId == 0x0AF12:
            return "AudioFire12"
        return "Generic FireWorks"

    def updateMatrixButton(self,a0):
        sender = self.sender()
        if a0:
            state = 1
        else:
            state = 0
        log.debug("set %s %d %d to %d" % (
                    self.MatrixButtonControls[sender][0],
                    self.MatrixButtonControls[sender][1],
                    self.MatrixButtonControls[sender][2],
                    state))
        self.hw.setMatrixMixerValue(self.MatrixButtonControls[sender][0], 
                                    self.MatrixButtonControls[sender][1],
                                    self.MatrixButtonControls[sender][2],
                                    state)

    def updateMatrixRotary(self,a0):
        sender = self.sender()
        vol = a0
        log.debug("set %s %d %d to %d" % (
                    self.MatrixRotaryControls[sender][0],
                    self.MatrixRotaryControls[sender][1],
                    self.MatrixRotaryControls[sender][2],
                    vol))
        self.hw.setMatrixMixerValue(self.MatrixRotaryControls[sender][0], 
                                    self.MatrixRotaryControls[sender][1],
                                    self.MatrixRotaryControls[sender][2],
                                    vol)

    def updateMatrixVolume(self,a0):
        sender = self.sender()
        vol = a0
        #vol = 0x01000000-vol
        log.debug("set %s %d %d to %d" % (
                    self.MatrixVolumeControls[sender][0],
                    self.MatrixVolumeControls[sender][1],
                    self.MatrixVolumeControls[sender][2],
                    vol))
        self.hw.setMatrixMixerValue(self.MatrixVolumeControls[sender][0], 
                                    self.MatrixVolumeControls[sender][1],
                                    self.MatrixVolumeControls[sender][2],
                                    vol)

    def updateVolume(self,a0):
        sender = self.sender()
        vol = a0
        #vol = 0x01000000-vol
        log.debug("set %s to %d" % (
                    self.VolumeControls[sender][0],
                    vol))
        self.hw.setContignuous(self.VolumeControls[sender][0],
                              vol)

    def updateSelector(self,a0):
        sender = self.sender()
        if a0:
            state = 1
        else:
            state = 0
        log.debug("set %s to %d" % (
                    self.SelectorControls[sender][0],
                    state))
        self.hw.setDiscrete(self.SelectorControls[sender][0], state)

    def updateTrigger(self):
        sender = self.sender()
        log.debug("trigger %s" % (self.TriggerControls[sender][0]))
        self.hw.setDiscrete(self.TriggerControls[sender][0], 1)

    def updateSPDIFmodeControl(self,a0):
        sender = self.sender()
        if a0:
            state = 1
        else:
            state = 0
        if state:
            log.debug("set %s to %d" % (
                        self.SPDIFmodeControls[sender][0],
                        self.SPDIFmodeControls[sender][1]))
            self.hw.setDiscrete(self.SPDIFmodeControls[sender][0], self.SPDIFmodeControls[sender][1])

    def updateDigIfaceControl(self, a0):
        sender = self.sender()
        state = a0
        # 0/2/3 is available but GUI set 0/1/2
        if a0 > 0:
            state += 1
        log.debug("set %s to %d" % (
            self.DigIfaceControls[sender][0], state))
        self.hw.setDiscrete(self.DigIfaceControls[sender][0], state)

    def updatePlbkRouteControl(self, src):
        sender = self.sender()
        path = self.PlbkRouteControls[sender][0]
        sink = self.PlbkRouteControls[sender][1]
        self.hw.setDiscrete(path, sink, src)
        self.setStreamLabel(src, sink)

    def setStreamLabel(self, src, sink):
        pos = src * 2 + 1
        self.StreamMonitors[sink].lblName.setText("Playback %d/%d" % (pos, pos + 1))

    def buildMixer(self):
        log.debug("Building mixer")
        self.MatrixButtonControls={}
        self.MatrixRotaryControls={}
        self.MatrixVolumeControls={}
        self.VolumeControls={}
        self.SelectorControls={}
        self.SPDIFmodeControls={}
        self.TriggerControls={}
        self.DigIfaceControls={}
        self.PlbkRouteControls={}
        self.StreamMonitors=[]

        nb_pys_out = self.hw.getDiscrete("/HwInfo/PhysicalAudioOutCount")
        nb_pys_in = self.hw.getDiscrete("/HwInfo/PhysicalAudioInCount")

        outputtabslayout = QHBoxLayout( self )
        outputtabs = QTabWidget(self)
        outputtabslayout.addWidget( outputtabs, 1 )

        for outpair in range(int(nb_pys_out/2)):
            tab = QWidget( outputtabs )
            tablayout = QHBoxLayout( tab )
            
            grpMonitor = QGroupBox(tab)
            tablayout.addWidget(grpMonitor)
    
            grpPlayback = QGroupBox(tab)
            tablayout.addWidget(grpPlayback)
    
            grpOutput = QGroupBox(tab)
            tablayout.addWidget(grpOutput)
    
            grpMonitor.setTitle("Monitor")
            grpPlayback.setTitle("Playback")
            grpOutput.setTitle("Output")
            
            # monitor controls
            grpMonitorLayout = QHBoxLayout()
            grpMonitor.setLayout(grpMonitorLayout);
            output_id = outpair * 2
            for inpair in range(int(nb_pys_in/2)):
                # create GUI elements
                strip = AfMonitorWidget( grpMonitor )
                grpMonitorLayout.addWidget( strip, 1 )
                input_id = inpair*2
                strip.lblName.setText("In %d/%d" % (input_id+1, input_id+2))

                # add the elements to the control structure

                self.MatrixButtonControls[strip.btnMute0] = ['/Mixer/MonitorMute', input_id, output_id]
                self.MatrixButtonControls[strip.btnMute1] = ['/Mixer/MonitorMute', input_id + 1, output_id + 1]
                self.MatrixButtonControls[strip.btnSolo0] = ['/Mixer/MonitorSolo', input_id, output_id]
                self.MatrixButtonControls[strip.btnSolo1] = ['/Mixer/MonitorSolo', input_id + 1, output_id + 1]
                self.MatrixRotaryControls[strip.rotPan0]  = ['/Mixer/MonitorPan', input_id, output_id]
                self.MatrixRotaryControls[strip.rotPan1]  = ['/Mixer/MonitorPan', input_id + 1, output_id + 1]
                self.MatrixVolumeControls[strip.sldGain0]  = ['/Mixer/MonitorGain', input_id, output_id]
                self.MatrixVolumeControls[strip.sldGain1]  = ['/Mixer/MonitorGain', input_id + 1, output_id + 1]

            # playback
            grpPlaybackLayout = QHBoxLayout()
            grpPlayback.setLayout(grpPlaybackLayout);
            strip = AfMonitorWidget( grpPlayback )
            grpPlaybackLayout.addWidget(strip, 1)
            strip.lblName.setText("Playback %d/%d" % (output_id+1, output_id+2))
            self.StreamMonitors.append(strip)

            self.VolumeControls[strip.sldGain0] = ["/Mixer/PC%dGain" % (output_id)]
            self.VolumeControls[strip.sldGain1] = ["/Mixer/PC%dGain" % (output_id+1)]
            self.SelectorControls[strip.btnMute0] = ["/Mixer/PC%dMute" % (output_id)]
            self.SelectorControls[strip.btnMute1] = ["/Mixer/PC%dMute" % (output_id+1)]
            self.SelectorControls[strip.btnSolo0] = ["/Mixer/PC%dSolo" % (output_id)]
            self.SelectorControls[strip.btnSolo1] = ["/Mixer/PC%dSolo" % (output_id+1)]

            # fix up mixer strip gui
            strip.rotPan0.hide()
            strip.rotPan1.hide()

            # output
            grpOutputLayout = QHBoxLayout()
            grpOutput.setLayout(grpOutputLayout);
            strip = AfMonitorWidget( grpOutput )
            grpOutputLayout.addWidget(strip, 1)
            strip.lblName.setText("Output %d/%d" % (output_id+1, output_id+2))

            self.VolumeControls[strip.sldGain0] = ["/Mixer/OUT%dGain" % (output_id)]
            self.VolumeControls[strip.sldGain1] = ["/Mixer/OUT%dGain" % (output_id+1)]
            self.SelectorControls[strip.btnMute0] = ["/Mixer/OUT%dMute" % (output_id)]
            self.SelectorControls[strip.btnMute1] = ["/Mixer/OUT%dMute" % (output_id+1)]
            self.SelectorControls[strip.btnSolo0] = ["/Mixer/OUT%dNominal" % (output_id)]
            self.SelectorControls[strip.btnSolo1] = ["/Mixer/OUT%dNominal" % (output_id+1)]

            # fix up mixer strip gui
            strip.btnSolo0.setText("Pad")
            strip.btnSolo1.setText("Pad")
            strip.rotPan0.hide()
            strip.rotPan1.hide()

            # add the tab
            outputtabs.addTab( tab, "Out %d/%d" % (output_id+1, output_id+2))

        # add an input config tab
        tab = QWidget( outputtabs )
        tablayout = QHBoxLayout( tab )

        for inpair in range(nb_pys_in):
            # create GUI elements
            log.debug("strip")
            grpInput = QGroupBox(tab)
            tablayout.addWidget(grpInput)
            grpInput.setTitle("In %d" % (inpair+1))

            grpInputLayout = QVBoxLayout()
            grpInput.setLayout(grpInputLayout);

            label = QLabel( grpInput )
            grpInputLayout.addWidget( label )
            label.setText("In %d" % (inpair+1))
            label.setAlignment(Qt.AlignCenter)

            btn = QToolButton( grpInput )
            grpInputLayout.addWidget( btn )
            btn.setText("Pad")
            btn.setCheckable(True)
            self.SelectorControls[btn] = ["/Mixer/IN%dNominal" % (inpair)]

            spacer = QSpacerItem(1,1,QSizePolicy.Minimum,QSizePolicy.Expanding)
            grpInputLayout.addItem(spacer)

        outputtabs.addTab( tab, "Input")

        # add an settings tab
        tab = QWidget( outputtabs )
        tablayout = QHBoxLayout( tab )
        outputtabs.addTab( tab, "Settings")
        settings = AfSettingsWidget( tab )

        has_sw_phantom = self.hw.getDiscrete("/HwInfo/PhantomPower")
        if has_sw_phantom:
            self.SelectorControls[settings.btnPhantom] = ["/PhantomPower"]
        else:
            settings.btnPhantom.hide()

        has_opt_iface = self.hw.getDiscrete('/HwInfo/OpticalInterface')
        if has_opt_iface:
            self.DigIfaceControls[settings.cmbDigIface] = ['/DigitalInterface']
        else:
            settings.DigIface.hide()

        has_plbk_route = self.hw.getDiscrete('/HwInfo/PlaybackRouting')
        if has_plbk_route:
            self.PlbkRouteControls[settings.cmbRoute1] = ['/PlaybackRouting', 0]
            self.PlbkRouteControls[settings.cmbRoute2] = ['/PlaybackRouting', 1]
            self.PlbkRouteControls[settings.cmbRoute3] = ['/PlaybackRouting', 2]
            if self.configrom.getModelId() == 0x0AF2:
                label_route2 = 'Headphone Out 1/2'
            else:
                label_route2 = 'Analog Out 3/4'
            settings.labelRoute2.setText(label_route2)
        else:
            settings.PlbkRoute.hide()

        self.TriggerControls[settings.btnSaveSettings] = ["/SaveSettings"]
        self.TriggerControls[settings.btnIdentify] = ["/Identify"]

        if self.configrom.getModelId() == 0x0AF12:
            settings.spdifMode.hide()
        else:
            self.SPDIFmodeControls[settings.radioConsumer] = ["/SpdifMode", 0]
            self.SPDIFmodeControls[settings.radioProfessional] = ["/SpdifMode", 1]

        # Store a reference to the "save" button for later manipulation
        self.btnSaveSettings = settings.btnSaveSettings

    def polledUpdate(self):
        ss = self.streamingstatus.selected()

        # Only alter controls sensitive to the streaming state when the
        # streaming state has changed.
        if (ss != self.streaming_state):
            ss_txt = self.streamingstatus.getEnumLabel(ss)
            # The device doesn't cope very well if "save settings" is done
            # while streaming is active
            self.btnSaveSettings.setEnabled(ss_txt=='Idle')

        self.streaming_state = ss

    def initValues(self):
        log.debug("Init values")

        for ctrl, info in self.MatrixVolumeControls.items():
            vol = self.hw.getMatrixMixerValue(self.MatrixVolumeControls[ctrl][0],
                                                self.MatrixVolumeControls[ctrl][1],
                                                self.MatrixVolumeControls[ctrl][2])

            #vol = 0x01000000-vol
            log.debug("%s volume is %d" % (ctrl.objectName() , vol))
            ctrl.setValue(vol)

            # connect the UI element
            ctrl.valueChanged.connect(self.updateMatrixVolume)

        for ctrl, info in self.MatrixButtonControls.items():
            state = self.hw.getMatrixMixerValue(self.MatrixButtonControls[ctrl][0],
                                                self.MatrixButtonControls[ctrl][1],
                                                self.MatrixButtonControls[ctrl][2])

            log.debug("%s state is %d" % (ctrl.objectName() , state))
            if state:
                ctrl.setChecked(True)
            else:
                ctrl.setChecked(False)

            # connect the UI element
            ctrl.clicked.connect(self.updateMatrixButton)

        for ctrl, info in self.MatrixRotaryControls.items():
            vol = self.hw.getMatrixMixerValue(self.MatrixRotaryControls[ctrl][0],
                                                self.MatrixRotaryControls[ctrl][1],
                                                self.MatrixRotaryControls[ctrl][2])

            log.debug("%s value is %d" % (ctrl.objectName(), vol))
            ctrl.setValue(vol)

            # connect the UI element
            ctrl.valueChanged.connect(self.updateMatrixRotary)

        for ctrl, info in self.VolumeControls.items():
            vol = self.hw.getContignuous(self.VolumeControls[ctrl][0])

            #vol = 0x01000000-vol
            log.debug("%s volume is %d" % (ctrl.objectName() , vol))
            ctrl.setValue(vol)

            # connect the UI element
            ctrl.valueChanged.connect(self.updateVolume)

        for ctrl, info in self.SelectorControls.items():
            state = self.hw.getDiscrete(self.SelectorControls[ctrl][0])
            log.debug("%s state is %d" % (ctrl.objectName() , state))
            if state:
                ctrl.setChecked(True)
            else:
                ctrl.setChecked(False)

            # connect the UI element
            ctrl.clicked.connect(self.updateSelector)

        for ctrl, info in self.TriggerControls.items():
            # connect the UI element
            ctrl.clicked.connect(self.updateTrigger)

        for ctrl, info in self.SPDIFmodeControls.items():
            state = self.hw.getDiscrete(self.SPDIFmodeControls[ctrl][0])
            log.debug("%s state is %d" % (ctrl.objectName() , state))
            if state == self.SPDIFmodeControls[ctrl][1]:
                ctrl.setChecked(True)
            else:
                ctrl.setChecked(False)

            # connect the UI element
            ctrl.toggled.connect(self.updateSPDIFmodeControl)

        for ctrl, info in self.DigIfaceControls.items():
            state = self.hw.getDiscrete(self.DigIfaceControls[ctrl][0])
            # 0/2/3 is available but GUI set 0/1/2
            if state > 0:
                state -= 1
            ctrl.setCurrentIndex(state)
            ctrl.activated.connect(self.updateDigIfaceControl)

        for ctrl, info in self.PlbkRouteControls.items():
            sink = self.PlbkRouteControls[ctrl][1]
            src = self.hw.getDiscrete(self.PlbkRouteControls[ctrl][0], sink)
            ctrl.setCurrentIndex(src)
            self.setStreamLabel(src, sink)
            ctrl.activated.connect(self.updatePlbkRouteControl)

        self.update_timer = QTimer(self)
        self.update_timer.timeout.connect(self.polledUpdate)
        self.update_timer.start(1000)
        self.streaming_state = -1

# vim: et



#
# Copyright (C) 2005-2008 by Daniel Wagner
#
# This file is part of FFADO
# FFADO = Free FireWire (pro-)audio drivers for Linux
#
# FFADO is based upon FreeBoB.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 2 of the License, or
# (at your option) version 3 of the License.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#

# from PyQt4.QtGui import QWidget
from ffado.import_pyqt import *

from ffado.config import *
import logging
log = logging.getLogger('bridgeco')

class BCoAudio5Control(QWidget):
    def __init__(self,parent = None):
        QWidget.__init__(self,parent)
        uicLoad("ffado/mixer/bcoaudio5", self)

        self.VolumeControls={
            'in_line12'  :   ['/Mixer/Feature_Volume_1', self.sldInput12],
            'in_line34'  :   ['/Mixer/Feature_Volume_2', self.sldInput34],
            'in_spdif'   :   ['/Mixer/Feature_Volume_3', self.sldInputSPDIF],
            'out_line12' :   ['/Mixer/Feature_Volume_6', self.sldOutput12],
            'out_line34' :   ['/Mixer/Feature_Volume_7', self.sldOutput34],
            'cross_a'    :   ['/Mixer/Feature_Volume_4', self.sldCrossA],
            'cross_b'    :   ['/Mixer/Feature_Volume_5', self.sldCrossB],
            }

        self.ComboControls={
            'line34source':   ['/Mixer/Selector_1', self.comboMixSource], 
        }

    def setComboMixSource(self,a0):
        self.setSelector('line34source', a0)

    def setVolumeIn12(self,a0):
        self.setVolume('in_line12', a0)

    def setVolumeIn34(self,a0):
        self.setVolume('in_line34', a0)

    def setVolumeInSPDIF(self,a0):
        self.setVolume('in_spdif', a0)

    def setVolumeOut12(self,a0):
        self.setVolume('out_line12', a0)

    def setVolumeOut34(self,a0):
        self.setVolume('out_line34', a0)

    def setCrossA(self,a0):
        self.setVolume('cross_a', a0)

    def setCrossB(self,a0):
        self.setVolume('cross_b', a0)

    def setVolume(self,a0,a1):
        name = a0
        vol = -a1
        log.debug("setting %s volume to %d" % (name, vol))
        self.hw.setContignuous(self.VolumeControls[name][0], vol)

    def setSelector(self,a0,a1):
        name = a0
        state = a1
        log.debug("setting %s state to %d" % (name, state))
        self.hw.setDiscrete(self.ComboControls[name][0], state)
        # verify
        state = self.hw.getDiscrete(self.ComboControls[name][0])
        self.hw.setDiscrete(self.ComboControls[name][0], state)

    def initValues(self):
        for name, ctrl in self.VolumeControls.items():
            vol = self.hw.getContignuous(ctrl[0])
            log.debug("%s volume is %d" % (name , vol))
            ctrl[1].setValue(-vol)

        for name, ctrl in self.ComboControls.items():
            state = self.hw.getDiscrete(ctrl[0])
            log.debug("%s state is %d" % (name , state))
            ctrl[1].setCurrentIndex( state )

# vim: et



#
# Copyright (C) 2017 by Jonathan Woithe
#
# This file is part of FFADO
# FFADO = Free FireWire (pro-)audio drivers for Linux
#
# FFADO is based upon FreeBoB.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#

# This module handles the importing of PyQt modules for both PyQt4 and PyQt5
# under Python2 or Python3.  If Python3 is installed it is assumed that
# PyQt5 is in use (this is reasonable because PyQt5 is what everyone wants
# to use under Python3).  Otherwise (that is, under Python2), an import of
# PyQt4 is tried first; if an import error occurs then PyQt5 is assumed.
#
# All modules used by any part of ffado-mixer are imported.  This greatly
# simplifies the process.  Otherwise the modules to import would be delivered
# by string variables, and there isn't a supported way to do this across 
# Python2 and Python3.

import sys
ffado_python3 = sys.version_info >= (3,)

if ffado_python3:
    ffado_pyqt_version = 5
else:
    try:
        from PyQt4 import QtGui
        ffado_pyqt_version = 4
    except ImportError:
        ffado_pyqt_version = 5

if ffado_pyqt_version == 4:
    from PyQt4 import QtGui, QtCore, Qt, uic
    from PyQt4.QtCore import QByteArray, QObject, QTimer, Qt, pyqtSignal, QString, pyqtSlot
    from PyQt4.QtGui import *
else:
    from PyQt5 import QtGui, Qt, QtCore, Qt, QtWidgets, uic
    from PyQt5.QtCore import QByteArray, QObject, pyqtSignal, pyqtSlot, QTimer, Qt
    from PyQt5.QtGui import *
    from PyQt5.QtWidgets import *



#
# Copyright (C) 2009-2010 by Arnold Krille
#
# This file is part of FFADO
# FFADO = Free FireWire (pro-)audio drivers for Linux
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 2 of the License, or
# (at your option) version 3 of the License.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#

# from PyQt4 import QtGui, QtCore, Qt
# from PyQt4.QtGui import QWidget, QScrollArea
from ffado.import_pyqt import *

import dbus

from ffado.widgets.matrixmixer import MatrixMixer
from ffado.widgets.crossbarrouter import *
from ffado.mixer.generic_dice_eap import *

from ffado.config import *

class BooleanControl:
    def __init__(self, hw, path):
        if bypassdbus:
            self.value = False
            return
        self.iface = dbus.Interface(
                hw.bus.get_object(hw.servername, path),
                dbus_interface="org.ffado.Control.Element.Boolean")
        self.value = self.iface.selected()

    def selected(self):
        return self.value

    def select(self, n):
        if bypassdbus:
            return False
        if self.iface.select(n):
            self.value = n
            return True
        return False

class DiscreteControl:
    def __init__(self, hw, path):
        if bypassdbus:
            self.value = 0
            return
        self.iface = dbus.Interface(
                hw.bus.get_object(hw.servername, path),
                dbus_interface="org.ffado.Control.Element.Discrete")
        self.value = self.iface.getValue()

    def getvalue(self):
        return self.value

    def setvalue(self, v):
        if v != self.value:
            self.iface.setValue(v)
            self.value = v

class Saffire_Dice(Generic_Dice_EAP):
    def __init__(self, parent=None):
        Generic_Dice_EAP.__init__(self, parent)

    def buildMixer(self):
        #print( self.hw )
        #print( self.hw.getText("/Generic/Nickname") )
        Generic_Dice_EAP.buildMixer(self)

        widget = QWidget()

        ModelName = self.configrom.getModelName().rstrip()
        if ffado.config.bypassdbus:
            # A hack for the bypassdbus case, where ModelName has a slightly
            # different format.  Rather than using the name as returned by
            # the device (eg: "SAFFIRE_PRO_40"), the name is taken from the
            # configuration file desciption (eg: "Saffire PRO 40").
            if ModelName[-2:] == "-1":
                ModelName = ModelName[:-2]
            if ModelName[-4:] == " DSP":
                ModelName = ModelName[:-4]
            ModelNum = ModelName[-2:]
            ModelName = "SAFFIRE_PRO"
            if ModelNum != "26":
                ModelName += "_"
            ModelName += ModelNum
            print( ModelName )

        if  ModelName == "SAFFIRE_PRO_14":
            uicLoad("ffado/mixer/Saffire_Pro14_monitoring.ui", widget)
        elif ModelName == "SAFFIRE_PRO_24" or self.configrom.getModelName() == "SAFFIRE_PRO_24DSP":
            uicLoad("ffado/mixer/Saffire_Pro24_monitoring.ui", widget)
        elif ModelName == "SAFFIRE_PRO26":
            uicLoad("ffado/mixer/Saffire_Pro26_monitoring.ui", widget)
        elif ModelName == "SAFFIRE_PRO_40":
            uicLoad("ffado/mixer/Saffire_Pro40_monitoring.ui", widget)

        # Add Monitoring to ffado-mixer panels
        scrollarea = QScrollArea(self.tabs)
        scrollarea.setWidgetResizable(False)
        scrollarea.setWidget(widget)
        self.tabs.addTab(scrollarea, "Monitoring")

        # Global settings
        self.muteInterface = BooleanControl(self.hw, self.hw.basepath+"/EAP/Monitoring/GlobalMute/State")
        widget.GlobalMute.setChecked(self.muteInterface.selected())
        widget.GlobalMute.toggled.connect(self.muteToggle)

        self.dimInterface = BooleanControl(self.hw, self.hw.basepath+"/EAP/Monitoring/GlobalDim/State")
        widget.GlobalDim.setChecked(self.dimInterface.selected())
        widget.GlobalDim.toggled.connect(self.dimToggle)
        self.dimLevelInterface = DiscreteControl(self.hw, self.hw.basepath+"/EAP/Monitoring/GlobalDim/Level")
        widget.DimLevel.setValue(self.dimLevelInterface.getvalue())
        widget.DimLevel.valueChanged.connect(self.dimLevelChanged)
        self.DimLevel = widget.DimLevel
        widget.DimLevel.setEnabled(self.dimInterface.selected())

        # Per line monitoring
        from collections import namedtuple
        LineInfo = namedtuple('LineInfo', ['widget','Interface'])
        self.nbLines = 4

        self.LineMonos = []
        if ModelName != "SAFFIRE_PRO26":
            # TODO: is this redundant on any other interface?
            # Mono/Stereo Switch
            p = LineInfo(widget.Mono_12, BooleanControl(self.hw, self.hw.basepath+"/EAP/Monitoring/Mono/Line1Line2"))
            self.LineMonos.append(p)
            p = LineInfo(widget.Mono_34, BooleanControl(self.hw, self.hw.basepath+"/EAP/Monitoring/Mono/Line3Line4"))
            self.LineMonos.append(p)

        # Volume Unactivating
        self.LineUnActivates = []
        p = LineInfo(widget.LineUnActivate_1, BooleanControl(self.hw, self.hw.basepath+"/EAP/Monitoring/LineOut/UnActivate1"))
        self.LineUnActivates.append(p)
        p = LineInfo(widget.LineUnActivate_2, BooleanControl(self.hw, self.hw.basepath+"/EAP/Monitoring/LineOut/UnActivate2"))
        self.LineUnActivates.append(p)
        p = LineInfo(widget.LineUnActivate_3, BooleanControl(self.hw, self.hw.basepath+"/EAP/Monitoring/LineOut/UnActivate3"))
        self.LineUnActivates.append(p)
        p = LineInfo(widget.LineUnActivate_4, BooleanControl(self.hw, self.hw.basepath+"/EAP/Monitoring/LineOut/UnActivate4"))
        self.LineUnActivates.append(p)

        # Mute
        self.LineMutes = []
        p = LineInfo(widget.LineMute_1, BooleanControl(self.hw, self.hw.basepath+"/EAP/Monitoring/LineOut/Mute1"))
        self.LineMutes.append(p)
        p = LineInfo(widget.LineMute_2, BooleanControl(self.hw, self.hw.basepath+"/EAP/Monitoring/LineOut/Mute2"))
        self.LineMutes.append(p)
        p = LineInfo(widget.LineMute_3, BooleanControl(self.hw, self.hw.basepath+"/EAP/Monitoring/LineOut/Mute3"))
        self.LineMutes.append(p)
        p = LineInfo(widget.LineMute_4, BooleanControl(self.hw, self.hw.basepath+"/EAP/Monitoring/LineOut/Mute4"))
        self.LineMutes.append(p)

        # per line Global Mute switch
        self.LineGMutes = []
        p = LineInfo(widget.LineGMute_1, BooleanControl(self.hw, self.hw.basepath+"/EAP/Monitoring/LineOut/GMute1"))
        self.LineGMutes.append(p)
        p = LineInfo(widget.LineGMute_2, BooleanControl(self.hw, self.hw.basepath+"/EAP/Monitoring/LineOut/GMute2"))
        self.LineGMutes.append(p)
        p = LineInfo(widget.LineGMute_3, BooleanControl(self.hw, self.hw.basepath+"/EAP/Monitoring/LineOut/GMute3"))
        self.LineGMutes.append(p)
        p = LineInfo(widget.LineGMute_4, BooleanControl(self.hw, self.hw.basepath+"/EAP/Monitoring/LineOut/GMute4"))
        self.LineGMutes.append(p)

        # per line Global Dim switch
        self.LineGDims = []
        p = LineInfo(widget.LineGDim_1, BooleanControl(self.hw, self.hw.basepath+"/EAP/Monitoring/LineOut/GDim1"))
        self.LineGDims.append(p)
        p = LineInfo(widget.LineGDim_2, BooleanControl(self.hw, self.hw.basepath+"/EAP/Monitoring/LineOut/GDim2"))
        self.LineGDims.append(p)
        p = LineInfo(widget.LineGDim_3, BooleanControl(self.hw, self.hw.basepath+"/EAP/Monitoring/LineOut/GDim3"))
        self.LineGDims.append(p)
        p = LineInfo(widget.LineGDim_4, BooleanControl(self.hw, self.hw.basepath+"/EAP/Monitoring/LineOut/GDim4"))
        self.LineGDims.append(p)

        # Volume
        self.LineVolumes = []
        p = LineInfo(widget.LineVolume_1, DiscreteControl(self.hw, self.hw.basepath+"/EAP/Monitoring/LineOut/Volume1"))
        self.LineVolumes.append(p)
        p = LineInfo(widget.LineVolume_2, DiscreteControl(self.hw, self.hw.basepath+"/EAP/Monitoring/LineOut/Volume2"))
        self.LineVolumes.append(p)
        p = LineInfo(widget.LineVolume_3, DiscreteControl(self.hw, self.hw.basepath+"/EAP/Monitoring/LineOut/Volume3"))
        self.LineVolumes.append(p)
        p = LineInfo(widget.LineVolume_4, DiscreteControl(self.hw, self.hw.basepath+"/EAP/Monitoring/LineOut/Volume4"))
        self.LineVolumes.append(p)
  
        if ModelName != "SAFFIRE_PRO_14":
            self.nbLines = 6
            if ModelName != "SAFFIRE_PRO26":
                # Mono/Stereo Switch
                p = LineInfo(widget.Mono_56, BooleanControl(self.hw, self.hw.basepath+"/EAP/Monitoring/Mono/Line5Line6"))
                self.LineMonos.append(p)
            # Volume Unactivating
            p = LineInfo(widget.LineUnActivate_5, BooleanControl(self.hw, self.hw.basepath+"/EAP/Monitoring/LineOut/UnActivate5"))
            self.LineUnActivates.append(p)
            p = LineInfo(widget.LineUnActivate_6, BooleanControl(self.hw, self.hw.basepath+"/EAP/Monitoring/LineOut/UnActivate6"))
            self.LineUnActivates.append(p)
            # Mute
            p = LineInfo(widget.LineMute_5, BooleanControl(self.hw, self.hw.basepath+"/EAP/Monitoring/LineOut/Mute5"))
            self.LineMutes.append(p)
            p = LineInfo(widget.LineMute_6, BooleanControl(self.hw, self.hw.basepath+"/EAP/Monitoring/LineOut/Mute6"))
            self.LineMutes.append(p)
            # per line Global Mute switch
            p = LineInfo(widget.LineGMute_5, BooleanControl(self.hw, self.hw.basepath+"/EAP/Monitoring/LineOut/GMute5"))
            self.LineGMutes.append(p)
            p = LineInfo(widget.LineGMute_6, BooleanControl(self.hw, self.hw.basepath+"/EAP/Monitoring/LineOut/GMute6"))
            self.LineGMutes.append(p)
            # per line Global Dim switch
            p = LineInfo(widget.LineGDim_5, BooleanControl(self.hw, self.hw.basepath+"/EAP/Monitoring/LineOut/GDim5"))
            self.LineGDims.append(p)
            p = LineInfo(widget.LineGDim_6, BooleanControl(self.hw, self.hw.basepath+"/EAP/Monitoring/LineOut/GDim6"))
            self.LineGDims.append(p)
            # Volume
            p = LineInfo(widget.LineVolume_5, DiscreteControl(self.hw, self.hw.basepath+"/EAP/Monitoring/LineOut/Volume5"))
            self.LineVolumes.append(p)
            p = LineInfo(widget.LineVolume_6, DiscreteControl(self.hw, self.hw.basepath+"/EAP/Monitoring/LineOut/Volume6"))
            self.LineVolumes.append(p)

        if ModelName == "SAFFIRE_PRO_40":
            self.nbLines = 10
            # Mono/Stereo Switch
            p = LineInfo(widget.Mono_78, BooleanControl(self.hw, self.hw.basepath+"/EAP/Monitoring/Mono/Line7Line8"))
            self.LineMonos.append(p)
            p = LineInfo(widget.Mono_910, BooleanControl(self.hw, self.hw.basepath+"/EAP/Monitoring/Mono/Line9Line10"))
            self.LineMonos.append(p)
            # Volume Unactivating
            p = LineInfo(widget.LineUnActivate_7, BooleanControl(self.hw, self.hw.basepath+"/EAP/Monitoring/LineOut/UnActivate7"))
            self.LineUnActivates.append(p)
            p = LineInfo(widget.LineUnActivate_8, BooleanControl(self.hw, self.hw.basepath+"/EAP/Monitoring/LineOut/UnActivate8"))
            self.LineUnActivates.append(p)
            p = LineInfo(widget.LineUnActivate_9, BooleanControl(self.hw, self.hw.basepath+"/EAP/Monitoring/LineOut/UnActivate9"))
            self.LineUnActivates.append(p)
            p = LineInfo(widget.LineUnActivate_10, BooleanControl(self.hw, self.hw.basepath+"/EAP/Monitoring/LineOut/UnActivate10"))
            self.LineUnActivates.append(p)
            # Mute
            p = LineInfo(widget.LineMute_7, BooleanControl(self.hw, self.hw.basepath+"/EAP/Monitoring/LineOut/Mute7"))
            self.LineMutes.append(p)
            p = LineInfo(widget.LineMute_8, BooleanControl(self.hw, self.hw.basepath+"/EAP/Monitoring/LineOut/Mute8"))
            self.LineMutes.append(p)
            p = LineInfo(widget.LineMute_9, BooleanControl(self.hw, self.hw.basepath+"/EAP/Monitoring/LineOut/Mute9"))
            self.LineMutes.append(p)
            p = LineInfo(widget.LineMute_10, BooleanControl(self.hw, self.hw.basepath+"/EAP/Monitoring/LineOut/Mute10"))
            self.LineMutes.append(p)
            # per line Global Mute switch
            p = LineInfo(widget.LineGMute_7, BooleanControl(self.hw, self.hw.basepath+"/EAP/Monitoring/LineOut/GMute7"))
            self.LineGMutes.append(p)
            p = LineInfo(widget.LineGMute_8, BooleanControl(self.hw, self.hw.basepath+"/EAP/Monitoring/LineOut/GMute8"))
            self.LineGMutes.append(p)
            p = LineInfo(widget.LineGMute_9, BooleanControl(self.hw, self.hw.basepath+"/EAP/Monitoring/LineOut/GMute9"))
            self.LineGMutes.append(p)
            p = LineInfo(widget.LineGMute_10, BooleanControl(self.hw, self.hw.basepath+"/EAP/Monitoring/LineOut/GMute10"))
            self.LineGMutes.append(p)
            # per line Global Dim switch
            p = LineInfo(widget.LineGDim_7, BooleanControl(self.hw, self.hw.basepath+"/EAP/Monitoring/LineOut/GDim7"))
            self.LineGDims.append(p)
            p = LineInfo(widget.LineGDim_8, BooleanControl(self.hw, self.hw.basepath+"/EAP/Monitoring/LineOut/GDim8"))
            self.LineGDims.append(p)
            p = LineInfo(widget.LineGDim_9, BooleanControl(self.hw, self.hw.basepath+"/EAP/Monitoring/LineOut/GDim9"))
            self.LineGDims.append(p)
            p = LineInfo(widget.LineGDim_10, BooleanControl(self.hw, self.hw.basepath+"/EAP/Monitoring/LineOut/GDim10"))
            self.LineGDims.append(p)
            # Volume
            p = LineInfo(widget.LineVolume_7, DiscreteControl(self.hw, self.hw.basepath+"/EAP/Monitoring/LineOut/Volume7"))
            self.LineVolumes.append(p)
            p = LineInfo(widget.LineVolume_8, DiscreteControl(self.hw, self.hw.basepath+"/EAP/Monitoring/LineOut/Volume8"))
            self.LineVolumes.append(p)
            p = LineInfo(widget.LineVolume_9, DiscreteControl(self.hw, self.hw.basepath+"/EAP/Monitoring/LineOut/Volume9"))
            self.LineVolumes.append(p)
            p = LineInfo(widget.LineVolume_10, DiscreteControl(self.hw, self.hw.basepath+"/EAP/Monitoring/LineOut/Volume10"))
            self.LineVolumes.append(p)

            # Adat/Spdif switch control from interface for Pro40
            self.adatSpdifInterface = BooleanControl(self.hw, self.hw.basepath+"/EAP/Monitoring/AdatSpdif/State")
            widget.AdatSpdif.setChecked(self.adatSpdifInterface.selected())
            widget.AdatSpdif.toggled.connect(self.adatSpdifToggle)

        if ModelName in ("SAFFIRE_PRO_14", "SAFFIRE_PRO_24"):
            # Line/Inst and Hi/Lo switches for Pro14 and 24
            widget.LineInSwitches.setVisible(True)
            self.LineInSwitches = []
            p = LineInfo(widget.LineInSwitchInst_1, BooleanControl(self.hw, self.hw.basepath+"/EAP/Monitoring/LineInstGain/LineInst1"))
            self.LineInSwitches.append(p)
            p = LineInfo(widget.LineInSwitchInst_2, BooleanControl(self.hw, self.hw.basepath+"/EAP/Monitoring/LineInstGain/LineInst2"))
            self.LineInSwitches.append(p)
            p = LineInfo(widget.LineInSwitchHi_3, BooleanControl(self.hw, self.hw.basepath+"/EAP/Monitoring/LineInstGain/LineGain3"))
            self.LineInSwitches.append(p)
            p = LineInfo(widget.LineInSwitchHi_4, BooleanControl(self.hw, self.hw.basepath+"/EAP/Monitoring/LineInstGain/LineGain4"))
            self.LineInSwitches.append(p)
            for i in range(4):
                self.LineInSwitches[i].widget.setChecked(self.LineInSwitches[i].Interface.selected())
                self.LineInSwitches[i].widget.toggled.connect(self.LineInSwitches[i].Interface.select)
            widget.LineInSwitchLine_1.setChecked(not self.LineInSwitches[0].Interface.selected())
            widget.LineInSwitchLine_2.setChecked(not self.LineInSwitches[1].Interface.selected())
            widget.LineInSwitchLo_3.setChecked(not self.LineInSwitches[2].Interface.selected())
            widget.LineInSwitchLo_4.setChecked(not self.LineInSwitches[3].Interface.selected())

        # Mono/Stereo Switch
        if ModelName != "SAFFIRE_PRO26":
            for i in range(int(self.nbLines/2)):
                self.LineMonos[i].widget.setChecked(self.LineMonos[i].Interface.selected())
                self.LineMonos[i].widget.toggled.connect(self.LineMonos[i].Interface.select)

        for i in range(self.nbLines):
            self.LineUnActivates[i].widget.setChecked(self.LineUnActivates[i].Interface.selected())
            self.LineUnActivates[i].widget.toggled.connect(self.LineUnActivates[i].Interface.select)
            self.LineMutes[i].widget.setChecked(self.LineMutes[i].Interface.selected())
            self.LineMutes[i].widget.toggled.connect(self.LineMutes[i].Interface.select)
            self.LineGMutes[i].widget.setChecked(self.LineGMutes[i].Interface.selected())
            self.LineGMutes[i].widget.toggled.connect(self.LineGMutes[i].Interface.select)
            self.LineGDims[i].widget.setChecked(self.LineGDims[i].Interface.selected())
            self.LineGDims[i].widget.toggled.connect(self.LineGDims[i].Interface.select)
            self.LineVolumes[i].widget.setValue(self.LineVolumes[i].Interface.getvalue())
            self.LineVolumes[i].widget.valueChanged.connect(self.LineVolumes[i].Interface.setvalue)
 
        # HW switch controls the possibility of monitoring each output separatly 
        widget.HWSwitch.setChecked(self.HWselected())
        widget.HWSwitch.toggled.connect(self.HWToggle)

        # Line Out monitoring enabling depends on H/W switch
        self.LineOut = widget.LineOut
        self.LineOut.setEnabled(self.HWselected())

    def muteToggle(self, mute):
        self.muteInterface.select(mute)
    def dimToggle(self, dim):
        self.dimInterface.select(dim)
        self.DimLevel.setEnabled(dim)
    def adatSpdifToggle(self, spdif):
        self.adatSpdifInterface.select(spdif)
        # FIXME
        # Here we should consider that router configuration has changed inside the device
        #   so that the device has to be rediscovered by the dbus (as it is the case when
        self.onSamplerateChange()

    def HWToggle(self, HW):
        for i in range(self.nbLines):
            self.LineUnActivates[i].widget.setChecked(not HW)
            self.LineMutes[i].widget.setChecked(False)
            self.LineGMutes[i].widget.setChecked(True)
            self.LineGDims[i].widget.setChecked(True)
            self.LineVolumes[i].widget.setValue(0)
        for i in self.LineMonos:
            i.widget.setChecked(False)
        self.LineOut.setEnabled(HW)

    def HWselected(self):
        selected = False
        for i in range(self.nbLines):
            if (not self.LineUnActivates[i].Interface.selected()):
                 selected = True
            if (self.LineMutes[i].Interface.selected()):
                 selected = True
        return selected

    def dimLevelChanged(self, value):
        self.dimLevelInterface.setvalue(value)
    def volumeChanged(self, value):
        self.volumeInterface.setvalue(value)

    def getDisplayTitle(self):
        return "Saffire PRO40/26/24/14 Mixer"

#
# vim: et ts=4 sw=4



#
# Copyright (C) 2014-2015 by Andras Muranyi
# Copyright (c) 2013 by Takashi Sakamoto (Yamaha GO Control)
# Copyright (C) 2005-2008 by Pieter Palmers
#
# This file is part of FFADO
# FFADO = Free FireWire (pro-)audio drivers for Linux
#
# FFADO is based upon FreeBoB.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 2 of the License, or
# (at your option) version 3 of the License.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#

# from PyQt4.QtGui import QWidget
from ffado.import_pyqt import *

from math import log10
from ffado.config import *

import logging
log = logging.getLogger('phase88')

class Phase88Control(QWidget):
    def __init__(self,parent = None):
        QWidget.__init__(self,parent)
        
    def initValues(self):

        uicLoad("ffado/mixer/phase88", self)

        self.VolumeControls={
            self.sldInputMasterL:   ['/Mixer/Feature_Volume_1', 1, self.sldInputMasterR, 2, self.linkButtonMaster, self.decibelMasterL, 0], 
            self.sldInputMasterR:   ['/Mixer/Feature_Volume_1', 2, self.sldInputMasterL, 1, self.linkButtonMaster, self.decibelMasterR, 0], 
            self.sldInput1:         ['/Mixer/Feature_Volume_2', 1, self.sldInput2, 2, self.linkButton12, self.decibel1, 0],
            self.sldInput2:         ['/Mixer/Feature_Volume_2', 2, self.sldInput1, 1, self.linkButton12, self.decibel2, 0],
            self.sldInput3:         ['/Mixer/Feature_Volume_3', 1, self.sldInput4, 2, self.linkButton34, self.decibel3, 0],
            self.sldInput4:         ['/Mixer/Feature_Volume_3', 2, self.sldInput3, 1, self.linkButton34, self.decibel4, 0],
            self.sldInput5:         ['/Mixer/Feature_Volume_4', 1, self.sldInput6, 2, self.linkButton56, self.decibel5, 0],
            self.sldInput6:         ['/Mixer/Feature_Volume_4', 2, self.sldInput5, 1, self.linkButton56, self.decibel6, 0],
            self.sldInput7:         ['/Mixer/Feature_Volume_5', 1, self.sldInput8, 2, self.linkButton78, self.decibel7, 0],
            self.sldInput8:         ['/Mixer/Feature_Volume_5', 2, self.sldInput7, 1, self.linkButton78, self.decibel8, 0],
            self.sldInputSPDIFL:    ['/Mixer/Feature_Volume_6', 1, self.sldInputSPDIFR, 2, self.linkButtonSPDIF, self.decibelSPDIFL, 0],
            self.sldInputSPDIFR:    ['/Mixer/Feature_Volume_6', 2, self.sldInputSPDIFL, 1, self.linkButtonSPDIF, self.decibelSPDIFR, 0],
            self.sldInputWavePlayL: ['/Mixer/Feature_Volume_7', 1, self.sldInputWavePlayR, 2, self.linkButtonWavePlay, self.decibelWavePlayL, 0],
            self.sldInputWavePlayR: ['/Mixer/Feature_Volume_7', 2, self.sldInputWavePlayL, 1, self.linkButtonWavePlay, self.decibelWavePlayR, 0]
            }

        self.SelectorControls={
            self.comboOutAssign:    '/Mixer/Selector_6', 
            self.comboInAssign:     '/Mixer/Selector_7', 
            self.comboExtSync:      '/Mixer/Selector_8', 
            self.comboSyncSource:   '/Mixer/Selector_9', 
            self.comboFrontBack:    '/Mixer/Selector_10'
        }

        self.MuteControls={
            self.muteButtonMasterL:   [self.sldInputMasterL, self.muteButtonMasterR],
            self.muteButtonMasterR:   [self.sldInputMasterR, self.muteButtonMasterL],
            self.muteButton1:         [self.sldInput1, self.muteButton2],
            self.muteButton2:         [self.sldInput2, self.muteButton1],
            self.muteButton3:         [self.sldInput3, self.muteButton4],
            self.muteButton4:         [self.sldInput4, self.muteButton3],
            self.muteButton5:         [self.sldInput5, self.muteButton6],
            self.muteButton6:         [self.sldInput6, self.muteButton5],
            self.muteButton7:         [self.sldInput7, self.muteButton8],
            self.muteButton8:         [self.sldInput8, self.muteButton7],
            self.muteButtonSPDIFL:    [self.sldInputSPDIFL, self.muteButtonSPDIFR],
            self.muteButtonSPDIFR:    [self.sldInputSPDIFR, self.muteButtonSPDIFL],
            self.muteButtonWavePlayL: [self.sldInputWavePlayL, self.muteButtonWavePlayR],
            self.muteButtonWavePlayR: [self.sldInputWavePlayR, self.muteButtonWavePlayL]
        }
        
        # gain control
        for ctl, params in self.VolumeControls.items():
                path    = params[0]
                idx     = params[1]
                dbmeter = params[5]
                
                #db = self.hw.getContignuous(path, idx)
                #vol = self.db2vol(db)
                vol = self.hw.getContignuous(path, idx)
                print("%s ch %d volume is %d" % (path, idx, vol))
                ctl.setValue(vol)
                dbmeter.setText(self.vol2dbstr(vol))
                self.VolumeControls[ctl][6] = vol
                
                pair    = params[2]
                pidx    = params[3]
                link    = params[4]
                
#               pdb = self.hw.getContignuous(path, pidx)
#               pvol = self.db2vol(db)
                pvol = self.hw.getContignuous(path, pidx)

                if pvol == vol:
                        link.setChecked(True)
                
                ctl.valueChanged.connect(self.updateVolume)

        # selector controls
        for ctl, param in self.SelectorControls.items():
                state = self.hw.getDiscrete(param)
                ctl.setCurrentIndex(state)
                
                ctl.activated.connect(self.updateSelector)

        # mute controls
        for ctl, param in self.MuteControls.items():
                ctl.toggled.connect(self.muteVolume)


    # helper functions 
    def muteVolume(self, state):
        sender    = self.sender()
        volctl    = self.MuteControls[sender][0]
        path      = self.VolumeControls[volctl][0]
        idx       = self.VolumeControls[volctl][1]
        pair      = self.VolumeControls[volctl][2]
        pidx      = self.VolumeControls[volctl][3]
        link      = self.VolumeControls[volctl][4]
        savedvol  = self.VolumeControls[volctl][6]
        psavedvol = self.VolumeControls[pair][6]

        if state == True :
            self.hw.setContignuous(path, -25600, idx) # The PHASE88 supports volume between 0 and -25600
            if link.isChecked():
                pair.setDisabled(state)
                self.MuteControls[sender][1].setChecked(state)
#                self.hw.setContignuous(path, 0, pidx)
        else:
            self.hw.setContignuous(path, savedvol, idx)
            if link.isChecked():
                pair.setDisabled(state)
                self.MuteControls[sender][1].setChecked(state)
#                self.hw.setContignuous(path, psavedvol, pidx)

    def updateVolume(self, vol):
        sender  = self.sender()
        path    = self.VolumeControls[sender][0]
        idx     = self.VolumeControls[sender][1]
        pair    = self.VolumeControls[sender][2]
        pidx    = self.VolumeControls[sender][3]
        link    = self.VolumeControls[sender][4]
        dbmeter = self.VolumeControls[sender][5]

        #db = self.vol2dbstr(vol)
        #self.hw.setContignuous(path, db, idx)
        self.hw.setContignuous(path, vol, idx)
        dbmeter.setText(self.vol2dbstr(vol))
        self.VolumeControls[sender][6] = vol

        if link.isChecked():
            pair.setValue(vol)

    def updateSelector(self, state):
        sender  = self.sender()
        path    = self.SelectorControls[sender]
        self.hw.setDiscrete(path, state)
#       if path == '/Mixer/Selector_7'
#            ctrl  = self.VolumeControls['line78']
#            vol = self.hw.getContignuous(ctrl[0])          # Recall volume for selected channel *******************************************************
#            name  = 'line78'
#            log.debug("%s volume is %d" % (name , vol))
#            ctrl[1].setValue(-vol)

    def vol2dbstr(self, vol):
        vol = vol + 25600
        if vol == 0 :
            return "- "+u"\u221E"+" dB"
        return str("{0:.2f}".format(log10( float(abs(vol) + 0.001) / 25600 ) * 20))+"dB"

# vim: et



#
# Copyright (C) 2008-2009 by Arnold Krille
#
# This file is part of FFADO
# FFADO = Free FireWire (pro-)audio drivers for Linux
#
# FFADO is based upon FreeBoB.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#

import os
import dbus

import shlex

import logging
log = logging.getLogger('configparser')

class DeviceList:
    def __init__( self, filename="" ):
        self.devices = list()
        if not filename == "":
            self.updateFromFile( filename )

    def updateFromFile( self, filename, must_exist=False):
        filename = os.path.expanduser(filename)
        log.debug("DeviceList::updateFromFile(%s)" % filename)
        if not os.path.exists( filename ):
            if must_exist:
                log.error("cannot open file")
            return
        f = open( filename, "r" )

        lex = Parser( f )
        config = {}
        while lex.peaktoken() != lex.eof:
            tmp = lex.parsenamedvalue()
            if tmp != None:
                config[ tmp[0] ] = tmp[1]
        if "device_definitions" in config:
            for dev in config["device_definitions"]:
                self.addDevice( dev )

    def getDeviceById( self, vendor, model ):
        if isinstance(vendor, dbus.Int32):
            vendor = str(int(vendor))
        if isinstance(model, dbus.Int32):
            model = str(int(model))

        log.debug("DeviceList::getDeviceById( %s, %s )" % (vendor, model ))
        for dev in self.devices:
            if int("%s" % dev['vendorid'], 0) == int("%s" % vendor, 0) and \
               int("%s" % dev['modelid'], 0) == int("%s" % model, 0):
                return dev
        tmp = dict()
        self.devices.append( tmp )
        return tmp

    def addDevice( self, device_dict ):
        log.debug("DeviceList::addDevice()")
        dev = self.getDeviceById( device_dict['vendorid'], device_dict['modelid'] )
        dev.update( device_dict )

class Parser:
    def __init__( self, file ):
        self.lex = shlex.shlex( file )
        self.eof = self.lex.eof

    def peaktoken( self ):
        token = self.lex.get_token()
        self.lex.push_token( token )
        return token

    def parselist( self, level="" ):
        token = self.peaktoken()
        if token != "(":
            return
        self.lex.get_token()
        log.debug("%sWill parse list" % level)
        ret = []
        token = self.peaktoken()
        while token != ")":
            ret.append( self.parsenamedvalue( level+"  " ) )
            token = self.peaktoken()
        log.debug("%slist is %s" % (level, str(ret)))
        self.lex.get_token()
        if self.peaktoken() == ",":
            log.debug("%sFound a delimiter" % level)
            self.lex.get_token()
        return ret

    def parsemap( self, level="" ):
        token = self.lex.get_token()
        if token != "{":
            return
        log.debug("%sWill parse map" % level)
        ret = {}
        token = self.peaktoken()
        while token != "}":
            #self.push_token( token )
            tmp = self.parsenamedvalue( level+"  " )
            if tmp != None:
                ret[ tmp[0] ] = tmp[1]
            token = self.peaktoken()
        token = self.lex.get_token()
        log.debug("%sMap ended with '%s' and '%s'"% (level,token,self.peaktoken()))
        if self.peaktoken() in (",",";"):
            log.debug("%sFound a delimiter!" % level)
            self.lex.get_token()
        return ret

    def parsevalue( self, level="" ):
        token = self.lex.get_token()
        log.debug("%sparsevalue() called on token '%s'" % (level, token))
        self.lex.push_token( token )
        if token == "(":
            value = self.parselist( level+"  " )
        elif token == "{":
            value = self.parsemap( level+"  " )
        else:
            value = self.lex.get_token().replace( "\"", "" )
        token = self.peaktoken()
        if token == ";":
            log.debug("%sFound a delimiter!" % level)
            self.lex.get_token()
        return value

    def parsenamedvalue( self, level="" ):
        token = self.lex.get_token()
        log.debug("%sparsenamedvalue() called on token '%s'" % (level, token))
        if token == "{":
            self.lex.push_token( token )
            return self.parsemap( level+"  " )
        if len(token) > 0 and token not in ("{","}","(",")",",",";"):
            log.debug("%sGot name '%s'" %(level,token))
            name = token
            token = self.lex.get_token()
            if token in ("=",":"):
                #print( "%sWill parse value" % level )
                value = self.parsevalue( level )
                return (name,value)
        log.debug("%sparsenamedvalue() will return None!" % level)
        return

#
# Have kind of a test directly included...
#
if __name__ == "__main__":

    import sys

    file = sys.argv[1]

    log.setLevel(logging.DEBUG)
    devs = DeviceList( file )

    print( devs.devices )

# vim: et



#
# Copyright (C) 2005-2008 by Pieter Palmers
#
# This file is part of FFADO
# FFADO = Free FireWire (pro-)audio drivers for Linux
#
# FFADO is based upon FreeBoB.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 2 of the License, or
# (at your option) version 3 of the License.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#

import logging
log = logging.getLogger('saffirebase')

# the class that holds all actual control code
class SaffireMixerBase:
    def __init__(self):
        pass

    def updateMatrixVolume(self,a0):
        sender = self.sender()
        vol = a0
        log.debug("set %s %d %d to %d" % (
                    self.VolumeControls[sender][0],
                    self.VolumeControls[sender][1],
                    self.VolumeControls[sender][2],
                    vol))
        self.hw.setMatrixMixerValue(self.VolumeControls[sender][0], 
                                    self.VolumeControls[sender][1],
                                    self.VolumeControls[sender][2],
                                    vol)
    def updateLowResVolume(self,a0):
        sender = self.sender()
        vol = 127 - a0
        log.debug("set %s to %d" % (
                    self.VolumeControlsLowRes[sender][0],
                    vol))
        self.hw.setDiscrete(self.VolumeControlsLowRes[sender][0], vol)

    def updateSelector(self,a0):
        sender = self.sender()
        if a0:
            state = 1
        else:
            state = 0
        log.debug("set %s to %d" % (
                    self.SelectorControls[sender][0],
                    state))
        self.hw.setDiscrete(self.SelectorControls[sender][0], state)

        # if there are linked selector controls, update them
        if len(self.SelectorControls[sender]) >= 2:
            linked = self.SelectorControls[sender][1]
            for ctl in linked:
                if ctl.isChecked():
                    state = 1
                else:
                    state = 0
                self.hw.setDiscrete(self.SelectorControls[ctl][0], state)

    def triggerButton(self):
        sender = self.sender()
        log.debug("trigger %s" % (
                    self.TriggerButtonControls[sender][0]))
        self.hw.setDiscrete(self.TriggerButtonControls[sender][0], 1)

    def saveText(self):
        sender = self.sender()
        textbox = self.saveTextControls[sender][0]
        log.debug("save %s" % (
                    textbox.text().ascii()))
        self.hw.setText(self.TextControls[textbox][0], textbox.text().ascii())

    def initCombo(self, combo):
        path = self.ComboControls[combo][0]
        combo.clear()
        nb_items = self.hw.enumCount(path)
        for i in range( nb_items ):
            combo.insertItem( nb_items, self.hw.enumGetLabel(path, i) )
        combo.setCurrentIndex( self.hw.enumSelected(path) )

    def selectCombo(self, mode):
        sender = self.sender()
        path = self.ComboControls[sender][0]
        self.hw.enumSelect(path, mode)
        sender.setCurrentIndex( self.hw.enumSelected(path) )

    def updateValues(self):
        for ctrl, info in self.VolumeControls.items():
            vol = self.hw.getMatrixMixerValue(self.VolumeControls[ctrl][0],
                                              self.VolumeControls[ctrl][1],
                                              self.VolumeControls[ctrl][2])
            log.debug("%s volume is %d" % (ctrl.objectName() , vol))
            ctrl.setValue(vol)
        for ctrl, info in self.VolumeControlsLowRes.items():
            vol = self.hw.getDiscrete(self.VolumeControlsLowRes[ctrl][0])

            log.debug("%s volume is %d" % (ctrl.objectName() , 127-vol))
            ctrl.setValue(127 - vol)

        for ctrl, info in self.SelectorControls.items():
            state = self.hw.getDiscrete(self.SelectorControls[ctrl][0])
            log.debug("%s state is %d" % (ctrl.objectName() , state))
            if state:
                ctrl.setChecked(True)
            else:
                ctrl.setChecked(False)

        for ctrl, info in self.TriggerButtonControls.items():
            pass

        for ctrl, info in self.TextControls.items():
            text = self.hw.getText(self.TextControls[ctrl][0])
            log.debug("%s text is %s" % (ctrl.objectName() , text))
            ctrl.setText(text)

        for ctrl, info in self.ComboControls.items():
            self.initCombo(ctrl)

    def polledUpdateVolumeLowRes(self, srcpath, ctrl, divider=1):
        vol = self.hw.getDiscrete(srcpath) / divider
        #log.debug("polledUpdateVolumeLowRes: vol = %s" % vol)
        ctrl.setValue(vol)

    def initValues(self):
        self.updateValues()
        for ctrl, info in self.VolumeControls.items():
            ctrl.valueChanged.connect(self.updateMatrixVolume)

        for ctrl, info in self.VolumeControlsLowRes.items():
            ctrl.valueChanged.connect(self.updateLowResVolume)

        for ctrl, info in self.SelectorControls.items():
            ctrl.stateChanged.connect(self.updateSelector)

        for ctrl, info in self.TriggerButtonControls.items():
            ctrl.clicked.connect(self.triggerButton)

        for ctrl, info in self.saveTextControls.items():
            ctrl.clicked.connect(self.saveText)

        for ctrl, info in self.ComboControls.items():
            ctrl.activated.connect(self.selectCombo)

# vim: et



#
# Copyright (C) 2008-2009 by Pieter Palmers
#               2009      by Arnold Krille
#
# This file is part of FFADO
# FFADO = Free FireWire (pro-)audio drivers for Linux
#
# FFADO is based upon FreeBoB.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#

import urllib
import os

# Python3 renamed ConfigParser to configparser.  Deal with this in a way
# which maintains compatibility with python2.
try:
    from configparser import ConfigParser
except ImportError:
    from ConfigParser import SafeConfigParser as ConfigParser

# The urlopen()/urlencode() functions from urllib in python2 are in 
# urllib.request and urllib.parse respectively under python2.
try:
    import urllib.request, urllib.parse
    url_newapi = 1
except ImportError:
    import urllib
    url_newapi = 0

from sys import version_info

from ffado.config import REGISTER_URL, INI_FILE_PATH, FFADO_CONFIG_DIR

# from PyQt4.QtGui import QMessageBox
# from PyQt4.QtCore import QByteArray
from ffado.import_pyqt import *

from ffado.regdialog import *

import logging
log = logging.getLogger('registration')

class ffado_registration:
    def __init__(self, ffado_version,
                       guid,
                       vendor_id,
                       model_id,
                       vendor_string,
                       model_string):
        # only use the section before the SVN mark
        # we don't need to keep track of all SVN version changes
        self.ffado_version = ffado_version.split('-')[0]
        self.guid = guid
        self.vendor_id = vendor_id
        self.model_id = model_id
        self.vendor_string = vendor_string
        self.model_string = model_string

        #check if config file path exists, if not, create it
        config_path = os.path.expanduser(FFADO_CONFIG_DIR)
        if not os.path.exists(config_path):
            os.makedirs(config_path)

        # parse the ini file
        self.config_filename = os.path.expanduser(INI_FILE_PATH)
        self.parser = ConfigParser()
        self.parser.read(self.config_filename)
        self.section_name = "%s:%X" % (self.ffado_version, self.guid)
        self.email = "(optional)"
        if self.parser.has_section("history") \
           and self.parser.has_option("history", "email"):
            self.email = self.parser.get("history", "email")

    def register_ffado_usage(self):
        post_vals = {}
        post_vals['guid'] = self.guid
        post_vals['vendor_id'] = self.vendor_id
        post_vals['model_id'] = self.model_id
        post_vals['vendor_string'] = self.vendor_string
        post_vals['model_string'] = self.model_string
        post_vals['ffado_version'] = self.ffado_version
        post_vals['email'] = self.email

        try:
            if url_newapi == 1:
                response = urllib.request.urlopen(REGISTER_URL,
                                                  urllib.parse.urlencode(post_vals).encode('ascii'))
            else:
                response = urllib.urlopen(REGISTER_URL,
                                          urllib.urlencode(post_vals))
        except:
            log.error("failed, network error")
            return (-1, "Network Error")
    
        lines = response.readlines()
        
        ok = False
        errline = "Bad response from server"
        for i in range(len(lines)):
            if lines[i][0:10] == b"RESULT: OK":
                ok = True
            elif lines[i][0:12] == b"RESULT: FAIL":
                ok = False
                if len(lines)>i+1:
                    errline = lines[i+1]
        if not ok:
            log.info("registration failed %s" % errline)
            return (-2, errline)
        else:
            return (0, "")
    
    def check_for(self, what):
        if not self.parser.has_section(self.section_name):
            return False
        if not self.parser.has_option(self.section_name, what):
            return False
        return self.parser.getboolean(self.section_name, what)
    
    def check_if_already_registered(self):
        return self.check_for("registered")
    
    def check_for_ignore(self):
        return self.check_for("ignore")
    
    def mark(self, what, value):
        if not self.parser.has_section(self.section_name):
            self.parser.add_section(self.section_name)
        self.parser.set(self.section_name, what, str(value))
    
    def mark_version_registered(self):
        self.mark("registered", True)
    
    def mark_ignore_version(self):
        self.mark("ignore", True)

    def remember_email(self, email):
        if not self.parser.has_section("history"):
            self.parser.add_section("history")
        self.parser.set("history", "email", str(email))

    def check_for_registration(self):

        if self.check_for_ignore():
            log.debug("user requested to ignore registration")
        else:
            if self.check_if_already_registered():
                log.debug("version/GUID combo already registered")
            else:
                log.debug("show dialog...")

                dlg = ffadoRegDialog(self.vendor_string, "0x%X" % self.vendor_id,
                                     self.model_string, "0x%X" % self.model_id,
                                     "0x%016X" % self.guid, self.ffado_version,
                                     self.email)
                dlg.exec_()

                if dlg.choice == "neversend":
                    self.mark_ignore_version()
                elif dlg.choice == "send":
                    if version_info[0] < 3:
                        # Python 2.x
                        asciiData = dlg.getEmail().toAscii()
                        self.email = asciiData.data()
                    else:
                        # Python 3 and above
                        self.email = ascii(dlg.getEmail())
                    self.remember_email(self.email)

                    retval = self.register_ffado_usage()
                    msg = QMessageBox()
                    if retval[0] == 0:
                        log.debug("registration successful")
                        devinfomsg = "<p>Device: %s %s<br> Vendor/Model Id: %X/%X<br>Device GUID: %016X<br>FFADO Version: %s<br>E-Mail: %s</p>" % \
                            (self.vendor_string, self.model_string, self.vendor_id, self.model_id, self.guid, self.ffado_version, self.email)
                        tmp = msg.question( msg, "Registration Successful",
                                            "<qt><b>Thank you.</b>" +
                                            "<p>The registration of the following information was successful:</p>" +
                                            devinfomsg +
                                            "</p>For this device you won't be asked to register again until you upgrade to a new version of FFADO.</p>",
                                            QMessageBox.Ok )
                        self.mark_version_registered()
                    else:
                        log.error("error: " + retval[1])
                        tmp = msg.question( msg, "Registration Failed", 
                                            "<qt><b>The registration at ffado.org failed.</b>" +
                                            "<p>Error message:</p><p>" + retval[1] +
                                            "</p><p>Try again next time?</p></qt>", 
                                            QMessageBox.Yes, QMessageBox.No )
                        if tmp == 4:
                            self.mark_ignore_version()
                elif dlg.choice == "nosend":
                    pass
        # write the updated config
        f = open(self.config_filename, "w+")
        self.parser.write(f)
        f.close()

# vim: et



#
# Copyright (C) 2005-2008 by Pieter Palmers
#
# This file is part of FFADO
# FFADO = Free FireWire (pro-)audio drivers for Linux
#
# FFADO is based upon FreeBoB.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 2 of the License, or
# (at your option) version 3 of the License.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#

# from PyQt4.QtGui import QWidget
from ffado.import_pyqt import *

from ffado.config import *

import logging
log = logging.getLogger('quatafire')

class QuataFire(QWidget):
    def __init__(self,parent = None):
        QWidget.__init__(self,parent)
        uicLoad("ffado/mixer/quatafire", self)

        self.VolumeControls={
                self.sldCh1: ['/Mixer/Feature_Volume_1', 1],
                self.sldCh2: ['/Mixer/Feature_Volume_1', 2],
                self.sldCh34: ['/Mixer/Feature_Volume_2', 0],
                self.sldCh56: ['/Mixer/Feature_Volume_3', 0],
                self.sldDawAll: ['/Mixer/Feature_Volume_4', 0],
                self.sldDawCH1: ['/Mixer/Feature_Volume_4', 1],
                self.sldDawCH2: ['/Mixer/Feature_Volume_4', 2],
                self.sldDawCH3: ['/Mixer/Feature_Volume_4', 3],
                self.sldDawCH4: ['/Mixer/Feature_Volume_4', 4],
                self.sldDawCH5: ['/Mixer/Feature_Volume_4', 5],
                self.sldDawCH6: ['/Mixer/Feature_Volume_4', 6],
                self.sldDawCH7: ['/Mixer/Feature_Volume_4', 7],
                self.sldDawCH8: ['/Mixer/Feature_Volume_4', 8],
                }
        self.PanControls={
                #self.dialCh1: ['/Mixer/Feature_Volume_1'],
                #self.dialCh2: ['/Mixer/Feature_Volume_1'],
                self.dialCh34: ['/Mixer/Feature_Volume_2'],
                self.dialCh56: ['/Mixer/Feature_Volume_3'],
                }

    def updateVolume(self,a0):
        sender = self.sender()
        vol = -a0
        log.debug("setting %s volume to %d" % (self.VolumeControls[sender][0], vol))
        self.hw.setContignuous(self.VolumeControls[sender][0], vol, self.VolumeControls[sender][1])

    def updatePan(self,a0):
        sender = self.sender()
        pan_left = a0
        if pan_left < 0:
            pan_left = 0

        pan_right = -a0
        if pan_right < 0:
            pan_right = 0

        log.debug("setting %s pan left to %d" % (self.PanControls[sender][0], -pan_left))
        self.hw.setContignuous(self.PanControls[sender][0], -pan_left, 1)
        log.debug("setting %s pan right to %d" % (self.PanControls[sender][0], -pan_right))
        self.hw.setContignuous(self.PanControls[sender][0], -pan_right, 2)

    def initValues(self):
        for ctrl, info in self.VolumeControls.items():
            vol = self.hw.getContignuous(self.VolumeControls[ctrl][0], self.VolumeControls[ctrl][1])
            val = -vol
            log.debug("%s volume is %d, set to %d" % (ctrl.objectName(), vol, val))
            ctrl.setValue(val)

            # connect the UI element
            ctrl.valueChanged.connect(self.updateVolume)

        for ctrl, info in self.PanControls.items():
            pan_left = self.hw.getContignuous(self.PanControls[ctrl][0], 1)
            pan_right = self.hw.getContignuous(self.PanControls[ctrl][0], 2)

            log.debug("%s pan left is %d" % (ctrl.objectName() , pan_left))
            log.debug("%s pan right is %d" % (ctrl.objectName() , pan_right))

            if pan_left == 0:
                val = pan_right
            else:
                val = -pan_left

            ctrl.setValue(val)
            # connect the UI element
            ctrl.valueChanged.connect(self.updatePan)

# vim: et



#
# Copyright (C) 2009 by Arnold Krille
#               2013 by Philippe Carriere
#
# This file is part of FFADO
# FFADO = Free FireWire (pro-)audio drivers for Linux
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 2 of the License, or
# (at your option) version 3 of the License.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#

# from PyQt4 import QtGui, QtCore
# from PyQt4.QtCore import pyqtSignal
# from PyQt4.QtGui import QFrame, QPainter, QGridLayout, QLabel, QComboBox
# from PyQt4.QtGui import QWidget, QVBoxLayout, QHBoxLayout, QPushButton
from ffado.import_pyqt import *

import dbus, math

import ffado.config

import logging
log = logging.getLogger("crossbarrouter")

class VuMeter(QFrame):
    def __init__(self, interface, output, input=None, parent=None):
        QFrame.__init__(self, parent)
        self.setLineWidth(1)
        self.setFrameStyle(QFrame.Panel|QFrame.Sunken)
        self.setMinimumSize(20, 20)

        self.level = 0

        self.interface = interface

        self.output = output

    def updateLevel(self, value):
        self.level = value
        self.update()

    def paintEvent(self, event):
        p = QPainter(self)
        value = self.level/4096
        r = self.rect()
        r.setHeight(int(r.height() * math.sqrt(value)))
        r.moveBottom(self.rect().height())
        p.fillRect(r, self.palette().highlight())

class OutputSwitcher(QFrame):
    """
The name is a bit misleading. This widget selectes sources for a specified
destination.

In mixer-usage this widget is at the top of the input-channel. Because the input
of the mixer is an available output from the routers point.
"""
    MixerRoutingChanged = pyqtSignal()
    def __init__(self, interface, outname, parent):
        QFrame.__init__(self, parent)
        self.interface = interface
        self.outname = outname
        self.lastin = ""

        self.setLineWidth(1)
        self.setFrameStyle(QFrame.Sunken|QFrame.Panel)

        self.layout = QGridLayout(self)
        self.setLayout(self.layout)

        self.lbl = QLabel(self.outname, self)
        self.lbl.setToolTip("The name of the destination that is to be controlled here.")
        self.layout.addWidget(self.lbl, 0, 0)

        self.vu = VuMeter(self.interface, outname, parent=self)
        self.layout.addWidget(self.vu, 0, 1)

        sources = self.interface.getSourceNames()

        self.combo = QComboBox(self)
        self.combo.setToolTip("<qt>Select the source for this destination.<br>Each destination can only receive sound from one source at a time. But one source can send sound to multiple destinations.</qt>")
        self.layout.addWidget(self.combo, 1, 0, 1, 2)
        self.combo.addItem("Disconnected")
        self.combo.addItems(sources)
        src = self.interface.getSourceForDestination(self.outname)
        self.lastin = str(src)
        if src != "":
            self.combo.setCurrentIndex(self.combo.findText(src))
        else:
            self.combo.setCurrentIndex(0)
        self.combo.activated[str].connect(self.comboCurrentChanged)


    def peakValue(self, value):
        self.vu.updateLevel(value)
        pass

    def comboCurrentChanged(self, inname):
        #log.debug("comboCurrentChanged( %s )" % inname)
        if inname == self.lastin:
            return
        if self.lastin != "":
            self.interface.setConnectionState(self.lastin, self.outname, False)

        if inname != "Disconnected":
            if self.interface.setConnectionState(str(inname), self.outname, True):
                if self.outname[:5] == "Mixer" or self.lastin[:5] == "Mixer" or str(inname)[:5] == "Mixer":
                    self.MixerRoutingChanged.emit()
                self.lastin = str(inname)
            else:
                log.warning(" Failed to connect %s to %s" % (inname, self.outname))
        else:
            self.lastin = ""


class CrossbarRouter(QWidget):
    MixerRoutingChanged = pyqtSignal(name='MixerRoutingChanged')
    def __init__(self, servername, basepath, parent=None):
        QWidget.__init__(self, parent);
        if not ffado.config.bypassdbus:
            self.bus = dbus.SessionBus()
            self.dev = self.bus.get_object(servername, basepath)
            self.interface = dbus.Interface(self.dev, dbus_interface="org.ffado.Control.Element.CrossbarRouter")
            self.destinations = self.interface.getDestinationNames()
        else:
            self.destinations = []

        self.settings = QtCore.QSettings(self)

        self.outgroups = []
        for ch in self.destinations:
            tmp = str(ch).split(":")[0]
            if not tmp in self.outgroups:
                self.outgroups.append(tmp)

        self.biglayout = QVBoxLayout(self)
        self.setLayout(self.biglayout)

        self.toplayout = QHBoxLayout()
        self.biglayout.addLayout(self.toplayout)

        self.vubtn = QPushButton("Switch peak meters", self)
        self.vubtn.setCheckable(True)
        self.vubtn.toggled.connect(self.runVu)
        self.toplayout.addWidget(self.vubtn)

        self.layout = QGridLayout()
        self.biglayout.addLayout(self.layout)

        self.switchers = {}
        for out in self.destinations:
            btn = OutputSwitcher(self.interface, out, self)
            self.layout.addWidget(btn, int(out.split(":")[-1]) + 1, self.outgroups.index(out.split(":")[0]))
            self.switchers[out] = btn
            self.switchers[out].MixerRoutingChanged.connect(self.updateMixerRouting)

        self.timer = QtCore.QTimer(self)
        self.timer.setInterval(200)
        self.timer.timeout.connect(self.updateLevels)

        if ffado_pyqt_version == 4:
            self.vubtn.setChecked(self.settings.value("crossbarrouter/runvu", False).toBool())
        else:
            self.vubtn.setChecked(self.settings.value("crossbarrouter/runvu", False) == u'true')

    def __del__(self):
        print( "CrossbarRouter.__del__()" )
        self.settings.setValue("crossbarrouter/runvu", self.vubtn.isChecked())

    def runVu(self, run=True):
        #log.debug("CrossbarRouter.runVu( %i )" % run)
        if run:
            self.timer.start()
        else:
            self.timer.stop()
            for sw in self.switchers:
                self.switchers[sw].peakValue(0)

    def updateLevels(self):
        #log.debug("CrossbarRouter.updateLevels()")
        if ffado.config.bypassdbus:
            return
        peakvalues = self.interface.getPeakValues()
        #log.debug("Got %i peaks" % len(peakvalues))
        for peak in peakvalues:
            #log.debug("peak = [%s,%s]" % (str(peak[0]),str(peak[1])))
            if peak[1] >= 0:
                self.switchers[peak[0]].peakValue(peak[1])

    def updateMixerRouting(self):
        self.MixerRoutingChanged.emit()

    def saveSettings(self, indent):
        routerSaveString = []
        routerSaveString.append('%s<dest_number>\n' % indent)
        routerSaveString.append('%s  %d\n' % (indent, len(self.destinations)))
        routerSaveString.append('%s</dest_number>\n' % indent)
        routerSaveString.append('%s<srcperdest>\n' % indent)
        routerSaveString.append('%s  2\n' % indent)
        routerSaveString.append('%s</srcperdest>\n' % indent)
        routerSaveString.append('%s<connections>\n' % indent)
        for out in self.destinations:
            routerSaveString.append('%s  ' % indent + out + ' ')
            routerSaveString.append(self.interface.getSourceForDestination(out) + '\n')
        routerSaveString.append('%s</connections>\n' % indent)
        return routerSaveString        

    def readSettings(self, routerReadString):
        sources = str(self.interface.getSourceNames())
        if routerReadString[0].find('<dest_number>') == -1:
            log.debug("Number of router destinations must be specified\n")
            return False
        if routerReadString[2].find('</dest_number>') == -1:
            log.debug("Incompatible xml file\n")
            return False
        n_dest = int(routerReadString[1])
        if n_dest != len(self.destinations):
            log.debug("Caution: numbers of destinations mismatch")
        if routerReadString[3].find('<srcperdest>') == -1:
            log.debug("Number of sources per destinations must be specified\n")
            return False
        if routerReadString[5].find('</srcperdest>') == -1:
            log.debug("Incompatible xml file\n")
            return False
        n_spd = int(routerReadString[4])
        if n_spd != 2:
            log.debug("Unable to handle more than one source for each destination;")
        try:
            idxb = routerReadString.index('<connections>')
            idxe = routerReadString.index('</connections>')
        except Exception:
            log.debug("Router connections not specified\n")
            idxb = -1
            idxe = -1
            return False
        if idxb >= 0:
            if idxe > idxb + 1:
                for s in routerReadString[idxb+1:idxe]:
                    destination = s.split()[0]
                    if str(self.destinations).find(destination) != -1:
                        source = s.split()[1]
                        if sources.find(source) != -1:                        
                            idx = self.switchers[destination].combo.findText(source)
                            self.switchers[destination].combo.setCurrentIndex(idx)
                            self.switchers[destination].comboCurrentChanged(source)
        return True
#
# vim: sw=4 ts=4 et



#
# Copyright (C) 2005-2008 by Pieter Palmers
#
# This file is part of FFADO
# FFADO = Free FireWire (pro-)audio drivers for Linux
#
# FFADO is based upon FreeBoB.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 2 of the License, or
# (at your option) version 3 of the License.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#

# from PyQt4.QtGui import QWidget
from ffado.import_pyqt import *

from ffado.config import *

class MackieOnyx(QWidget):
    def __init__(self,parent = None):
        QWidget.__init__(self,parent)
        uicLoad("ffado/mixer/mackieonyx", self)

    def initValues(self):
        pass

# vim: et



#
# Copyright (C) 2009-2010 by Arnold Krille
#               2013 by Philippe Carriere
#
# This file is part of FFADO
# FFADO = Free FireWire (pro-)audio drivers for Linux
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 2 of the License, or
# (at your option) version 3 of the License.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#

# from PyQt4 import QtGui, QtCore, Qt
# from PyQt4.QtGui import QWidget, QGridLayout, QTabWidget, QScrollArea
from ffado.import_pyqt import *

import dbus

from ffado.widgets.matrixmixer import MatrixMixer
from ffado.widgets.crossbarrouter import *

from ffado.config import *

class Generic_Dice_EAP(QWidget):
    def __init__(self, parent=None):
        QWidget.__init__(self, parent)
        self.layout = QGridLayout(self)
        self.setLayout(self.layout)
        self.tabs = QTabWidget(self)
        self.tabs.setTabPosition(QTabWidget.West)
        self.layout.addWidget(self.tabs)

    def buildMixer(self):
        #print( self.hw )
        #print( self.hw.getText("/Generic/Nickname") )
        self.mixer = MatrixMixer(self.hw.servername, self.hw.basepath+"/EAP/MatrixMixer", self, "Columns_are_outputs", -1, None, None, False, QTabWidget.North, QTabWidget.Rounded)
        self.tabs.addTab(self.mixer, "Mixer")

        self.router_scrollarea = self.buildRouter(self.hw.servername, self.hw.basepath+"/EAP/Router")
        self.tabs.addTab(self.router_scrollarea, "Crossbar Router")

    def buildRouter(self, servername, path):
        self.router = CrossbarRouter(servername, path, self)
        self.router.MixerRoutingChanged.connect(self.mixer.updateRouting)
        scrollarea = QScrollArea(self.tabs)
        scrollarea.setWidgetResizable(True)
        scrollarea.setWidget(self.router)
        return scrollarea

    def onSamplerateChange(self):
        # Router configuration is samplerate dependent for DICE EAP devices
        # Destroy and redraw the crossbar router view when called
        n = self.tabs.indexOf(self.router_scrollarea)
        self.tabs.removeTab(n)
        self.router.destroy()
        self.router_scrollarea.destroy()

        self.router_scrollarea = self.buildRouter(self.hw.servername, self.hw.basepath+"/EAP/Router")
        self.tabs.insertTab(n, self.router_scrollarea, "Crossbar Router")
        self.tabs.setCurrentWidget(self.router_scrollarea)

        self.mixer.updateRouting()

    def saveSettings(self, indent):
        saveString = []
        idt = indent + "  "
        saveString.append('%s<mixer>\n' % indent)
        saveString.extend(self.mixer.saveSettings(idt))
        # Do not forget to mention the adopted rule for matrix columns mixer
        #  This might be useful for future import function
        saveString.append("%s  <col_rule>\n" % indent)
        saveString.append("%s    Columns_are_outputs\n" % indent)
        saveString.append("%s  </col_rule>\n" % indent)
        saveString.append('%s</mixer>\n' % indent)
        saveString.append('%s<router>\n' % indent)
        saveString.extend(self.router.saveSettings(idt))
        saveString.append('%s</router>\n' % indent)
        return saveString

    def readSettings(self, readString):
        try:
            idxb = readString.index('<mixer>')
            idxe = readString.index('</mixer>')
        except Exception:
            log.debug("No mixer settings found")
            idxb = -1
            idxe = -1
        if idxb >= 0:
            if idxe > idxb + 1:
                stringMixer = []
                for s in readString[idxb+1:idxe]:
                    stringMixer.append(s)
                # When trying to import from a different device, the rule for column interpretation is
                # not necessarily the same
                try:
                    idx = stringMixer.index('<col_rule>')
                except Exception:
                    log.debug('Do not know how to handle column versus input/output')
                    idx = -1
                transpose_coeff = False
                if idx >=0:
                    if stringMixer[idx+1].find("Columns_are_outputs") == -1:
                        log.debug('Transposing the matrix coefficient; you are importing, are not you ?')
                        transpose_coeff = True
                if self.mixer.readSettings(stringMixer, transpose_coeff):
                    log.debug("Mixer settings modified")
                del stringMixer
        try:
            idxb = readString.index('<router>')
            idxe = readString.index('</router>')
        except Exception:
            log.debug("No router settings found")
            idxb = -1
            idxe = -1
        if idxb >= 0:
            if idxe > idxb + 1:
                stringRouter = []
                for s in readString[idxb+1:idxe]:
                    stringRouter.append(s)
                if self.router.readSettings(stringRouter):
                    log.debug("Router settings modified")
                del stringRouter

    #def getDisplayTitle(self):
    #    return "Saffire PRO40/PRO24 Mixer"

#
# vim: et ts=4 sw=4



# -*- coding: utf-8 -*-
#
# Copyright (C) 2005-2008 by Pieter Palmers
#
# This file is part of FFADO
# FFADO = Free FireWire (pro-)audio drivers for Linux
#
# FFADO is based upon FreeBoB.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 2 of the License, or
# (at your option) version 3 of the License.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#

# from PyQt4.QtGui import QDialog
from ffado.import_pyqt import *

from ffado.config import *

import logging
log = logging.getLogger('registration')

REGISTRATION_MESSAGE = """
<html><head><meta name="qrichtext" content="1" />
<style type="text/css">\np, li { white-space: pre-wrap; }\n</style></head>
<body style=" font-family:'Sans Serif'; font-size:9pt; font-weight:400; font-style:normal;">\n
<p style=" margin-top:12px; margin-bottom:12px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;">
You are running this version of FFADO for the first time with this device. </p>
<p style=" margin-top:12px; margin-bottom:12px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;">If FFADO is useful to you, please consider a donation through <a href="http://www.ffado.org/?q=donations ">http://www.ffado.org/?q=donations</a>.</p>
<p style=" margin-top:12px; margin-bottom:12px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;">
In order to collect usage statistics we would like to send some information about your system to ffado.org.
It is very important for us to have good usage statistics. This is to convince vendors that Linux users 
do exist and is where you as a user can help the project. </p>
<p style=" margin-top:12px; margin-bottom:12px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;">
The information collected is intended only for usage monitoring. The email address is optional, and will 
be used for FFADO related announcements only. If you provide one, please provide a valid one. </p>
<p style=" margin-top:12px; margin-bottom:12px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;">
Note: This registration can also be performed on-line at 
<a href="http://www.ffado.org/?q=usage">http://www.ffado.org/?q=usage</a>.</p></body></html>
"""

class ffadoRegDialog(QDialog):
    def __init__(self, vendor_name, vendor_id, model_name, model_id,
                 guid, version, email="(optional)",
                 parent = None):
        QDialog.__init__(self,parent)
        uicLoad("ffado/regdialog", self)
        self.txtVendorName.setText(vendor_name)
        self.txtVendorId.setText(vendor_id)
        self.txtModelName.setText(model_name)
        self.txtModelId.setText(model_id)
        self.txtGUID.setText(guid)
        self.txtVersion.setText(version)
        self.txtEmail.setText(email)
        self.txtMessage.setHtml(REGISTRATION_MESSAGE)

        self.choice = "nosend"
        self.btnSend.clicked.connect(self.buttonPressed)
        self.btnNoSend.clicked.connect(self.buttonPressed)
        self.btnNeverSend.clicked.connect(self.buttonPressed)

    def buttonPressed(self):
        sender = self.sender()
        if sender == self.btnSend:
            log.debug("user chose to send")
            self.choice = "send"
        elif sender ==  self.btnNoSend:
            log.debug("user chose not to send")
            self.choice = "nosend"
        elif sender ==  self.btnNeverSend:
            log.debug("user chose to never send")
            self.choice = "neversend"
        self.close()

    def getEmail(self):
        return self.txtEmail.text()

# vim: et
